import sys
sys.path.insert(0, './')
sys.path.insert(0, '../')

import os
import time
from flask import Flask, render_template, request,redirect
from flask_flatpages import FlatPages
import base64,struct,hashlib
from cloudmesh.cloudmesh import cloudmesh
from datetime import datetime
from cloudmesh.cm_config import cm_config
from datetime import datetime
try:
    from sh import xterm
except:
    print "xterm not suppported"
    #TODO: THERE SHOULD BE A VARIABLE SET HERE SO THAT THE ARROW START UP BUTTON 
    #      CAN RETURN MEANINGFULL MESSAGE IF NOT SUPPORTED
import yaml

######################################################################
# setting up reading path for the use of yaml
######################################################################

with_write = True


######################################################################
# setting up view state and way to access viewstate
######################################################################
view_state  = {'table':{}}


def v(names):
    dict = view_state  
    print view_state  
    try:
        for name in names:
            dict = dict[name]
        return dict
    except :
        return ""


######################################################################
# setting up reading path for the use of yaml
######################################################################

default_path = '.futuregrid/cloudmesh.yaml'
home = os.environ['HOME']
filename = "%s/%s" % (home, default_path)

######################################################################
# global vars
######################################################################

DEBUG = True
FLATPAGES_AUTO_RELOAD = DEBUG
FLATPAGES_EXTENSION = '.md'

"""
import pkg_resources
version = pkg_resources.get_distribution("flask_cm").version
"""
version = "0.7.2"


clouds = cloudmesh()
clouds.refresh()

clouds.refresh_user_id()

prefix = clouds.prefix()

######################################################################
# STARTING THE FLASK APP
######################################################################

app = Flask(__name__)
app.config.from_object(__name__)
pages = FlatPages(app)

######################################################################
# ACTIVATE STRUCTURE
######################################################################


def make_active(name):
    active = {'home': "",
              'table': "",
              'contact': "",
              'flavors': "",
              'images': "",
              'metric': "",
              'profile': "",
              'vm_info': "",
              'projects': "",
              'security': "",
              'updatekeypair':"",
              'clouds':""}
    
    active[name] = 'active'
    return active

######################################################################
# ROUTE: /
######################################################################


@app.route('/')
def index():
    active = make_active('home')
    return render_template('index.html',
                           pages=pages,
                           active=active,
                           version=version)

######################################################################
# ROUTE: REFRESH
######################################################################


@app.route('/cm/refresh/')
@app.route('/cm/refresh/<cloud>/')
def refresh(cloud=None, server=None):
    print "-> refresh", cloud, server
    clouds.refresh()
    return table()

######################################################################
# ROUTE: Filter
######################################################################


@app.route('/cm/filter/<cloud>/',methods=['GET','POST'])
def filter(cloud=None):
    print "-> filter", cloud
    filterIds = []
    do_status_filter = False
    # DOCUMENTATION ABOUT WHERE AND HOW view_state is managed is missing
    if cloud==None and view_state.has_key('table') :
        allClouds = clouds.active()
        # VARIABLE NAME POOR CHOICE, CONFUSING
        for clud in allClouds:
            do_status_filter_inner = False
            filterIds = []
            if view_state['table'].has_key(clud):
                print 'filtering',clud
                ACTIVE = view_state['table'][clud].has_key('ACTIVE')
                STOPPED = view_state['table'][clud].has_key('STOPPED')
                ERROR = view_state['table'][clud].has_key('ERROR')
                SHUTOFF = view_state['table'][clud].has_key('SHUTOFF')
                ME = view_state['table'][clud].has_key('ME')
                if(ACTIVE or STOPPED or ERROR or SHUTOFF):
                            do_status_filter_inner = True
                if(ACTIVE or STOPPED or ERROR or SHUTOFF or ME) :
                    f_cloud = clouds.clouds[clud]
                    for id, server in f_cloud['servers'].iteritems():
                        if do_status_filter_inner and not server['status'] in request.form : 
                            filterIds.append(id)
                        elif  (ME and not server['user_id'] == f_cloud['user_id'] ) :
                            filterIds.append(id)
                    for id in filterIds :
                        del f_cloud['servers'][id]
            
    elif request.method == 'POST':
        
        ACTIVE = 'ACTIVE' in request.form
        STOPPED = 'STOPPED'in request.form
        ERROR = 'ERROR'in request.form
        SHUTOFF = 'SHUTOFF'in request.form
        ME = 'ME'in request.form
        view_state['table'][cloud] = {}
        #IF STATES WOULD BE IN A DICT, WE COULD NICELY ITTERATE OVER THEM
        if ACTIVE:
            view_state['table'][cloud]['ACTIVE'] = 'checked'
        if STOPPED:
            view_state['table'][cloud]['STOPPED'] = 'checked'
        if ERROR:
            view_state['table'][cloud]['ERROR'] = 'checked'
        if SHUTOFF:
            view_state['table'][cloud]['SHUTOFF'] = 'checked'
        if ME:
            view_state['table'][cloud]['ME'] = 'checked'
        
        clouds.refresh(names = [cloud])
        
        if(ACTIVE or STOPPED or ERROR or SHUTOFF):
            do_status_filter = True
        
        if(ACTIVE or STOPPED or ERROR or SHUTOFF or ME) :
            f_cloud = clouds.clouds[cloud]
            for id, server in f_cloud['servers'].iteritems():
                if do_status_filter and not server['status'] in request.form : 
                    filterIds.append(id)
                elif  (ME and not server['user_id'] == f_cloud['user_id'] ) :
                    filterIds.append(id)
            for id in filterIds :
                del f_cloud['servers'][id]
        
    return redirect("/table/")


######################################################################
# ROUTE: KILL
######################################################################


@app.route('/cm/kill/')
def kill_vms():
    print "-> kill all"
    r = cm("--set", "quiet", "kill", _tty_in=True)
    return table()

######################################################################
# ROUTE: DELETE
######################################################################


@app.route('/cm/delete/<cloud>/<server>/')
def delete_vm(cloud=None, server=None):
    print "-> delete", cloud, server
    # if (cloud == 'india'):
    #  r = cm("--set", "quiet", "delete:1", _tty_in=True)
    clouds.delete(cloud, server)
    time.sleep(5)
    clouds.refresh()
    return table()

######################################################################
# ROUTE: DELETE GROUP
######################################################################
@app.route('/cm/delete/<cloud>/')
def delete_vms(cloud=None):
# donot do refresh before delete, this will cause all the vms to get deleted  
    f_cloud = clouds.clouds[cloud]
    for id, server in f_cloud['servers'].iteritems():   
        print "-> delete", cloud, id
        clouds.delete(cloud, id)
    time.sleep(7)
    f_cloud['servers'] = {}
    return redirect("/table/")


######################################################################
# ROUTE: ASSIGN PUBLIC IP
######################################################################


@app.route('/cm/assignpubip/<cloud>/<server>/')
def assign_public_ip(cloud=None, server=None):
    config = cm_config()
    dict_t = config.get()
    try :
        if dict_t['clouds'][cloud]['cm_automatic_ip'] == False:
            clouds.assign_public_ip(cloud,server)
            clouds.refresh(names = [cloud])
            return redirect("/table/")
        else:
            return "Manual public ip assignment is not allowed for "+cloud+" cloud"
    except Exception, e:
         return str(e) + "Manual public ip assignment is not allowed for "+cloud+" cloud"
######################################################################
# ROUTE: START
######################################################################

#
# WHY NOT USE cm_keys as suggested?
#
@app.route('/cm/start/<cloud>/')
def start_vm(cloud=None, server=None):
    print "*********** STARTVM", cloud
    print "-> start", cloud
    # if (cloud == 'india'):
    #  r = cm("--set", "quiet", "start:1", _tty_in=True)
    key = None
    config = cm_config()
    dict_t = config.get()
    if dict_t.has_key('keys'):
        key = dict_t['keys']['default']
    d = clouds.default(cloud)
    vm_flavor = d['flavor']
    vm_image = d['image']
    
    clouds.create(cloud, prefix, "001", vm_image, vm_flavor, key)
    return table()

'''
#gregorss test
@app.route('/cm/metric/<startdate>/<enddate>/<host>')
def list_metric(cloud=None, server=None):
    print "-> generate metric", startdate, endadte
    #r = fg-metric(startdate, enddate, host, _tty_in=True)
    return render_template('metric1.html',
                           startdate=startdate,
                           active=active,
                           version=version,
                           endate=enddate)
    #return table()
'''

######################################################################
# ROUTE: SAVE
######################################################################


@app.route('/save/')
def save():
    print "Saving the cloud status"
    clouds.save()
    return table()

######################################################################
# ROUTE: LOAD
######################################################################


@app.route('/load/')
def load():
    print "Loading the cloud status"
    clouds.load()
    return table()

######################################################################
# ROUTE: TABLE
######################################################################


@app.route('/table/')
def table():
    active_clouds = clouds.active()

    active = make_active('table')
    time_now = datetime.now().strftime("%Y-%m-%d %H:%M")
    filter()
    # keys = clouds.get_keys()
    return render_template('table.html',
                           updated=time_now,
                           keys="",  # ",".join(clouds.get_keys()),
                           clouds=clouds.clouds,
                           order=active_clouds,
                           image='myimage',
                           pages=pages,
                           active=active,
                           version=version,
                           v = v,
                           dir = dir)



######################################################################
# ROUTE: PROJECTS
######################################################################
#
# WHY NOT USE cm_projects as suggested, putting everything in one code becomes confusing and if separated 
#  increases code maintainability and readability
#
def set_default_project(name, project_names, type):
    global default_project;
    default_project = name
    selected = {}
    for name in project_names:
        selected[name] = ""
    selected[default_project] = type
    print selected
    return selected

def buildProjectNamesArray(projects):
     project_names=[]
     for project_name in projects:
        project_names.append(project_name);
     return project_names;

@app.route('/projects/', methods=['GET','POST'])
def display_project():
    global default_project;
    

    ############reading from yaml file ############
    config_project = cm_config()
    activeProjects=config_project.projects('active')
    project_names=buildProjectNamesArray(activeProjects)
    activeClouds=config_project.active()
    
    configurations= config_project.get() 
    default_project=configurations['projects']['default']
    selected=set_default_project(default_project, project_names,'checked')
     ############  end of reading from yaml file ############

    active = make_active('projects')
    
    if request.method == 'POST':
        radioSelected={}
        for cloud in activeClouds:
            if 'openstack' in cloud:
                default_project= request.form['selected_project'] 
                print default_project
                ############ writing in yaml file ############
                yamlFile= config_project.get();
                yamlFile['clouds'][cloud]['default']['project']=default_project;
                yamlFile['projects']['default']=default_project
                write_yaml(filename,yamlFile)
                ############ end of writing in yaml file ############
                selected = set_default_project(default_project, project_names,'checked')


    return redirect("/profile/")

######################################################################
# ROUTE: VM Login
######################################################################


@app.route('/cm/login/<cloud>/<server>/')
def vm_login(cloud=None,server=None):
    global clouds
    message = ''
    active = make_active('vm_login')
    time_now = datetime.now().strftime("%Y-%m-%d %H:%M")
    
    server=clouds.clouds[cloud]['servers'][server]
     
    if len(server['addresses'][server['addresses'].keys()[0]]) < 2:  
        mesage = 'Cannot Login Now, Public IP not assigned'
        print message
        
    else :
        message = 'Logged in Successfully'
        ip = server['addresses'][server['addresses'].keys()[0]][1]['addr'] 
        # THIS IS A BUG AND MUST BE SET PER VM, E.G. sometimesvm type probably decides that?
        print "ssh",'ubuntu@'+ip
        xterm('-e','ssh','ubuntu@'+ip,_bg=True)
        
    return redirect("/table/")
######################################################################
# ROUTE: VM INFO
######################################################################


@app.route('/cm/info/<cloud>/<server>/')
def vm_info(cloud=None,server=None):

    active = make_active('vm_info')
    time_now = datetime.now().strftime("%Y-%m-%d %H:%M")
    
    return render_template('vm_info.html',
                           updated=time_now,
                           keys="", 
                           server=clouds.clouds[cloud]['servers'][server],
                           id = server,
                           cloudname = cloud, 
                           active=active,
                           version=version,
                           table_fun=maketablefromdict )
                        
def maketablefromdict(the_dict):
    return_str = ''
    if isinstance(the_dict, dict):
        for name,value in the_dict.iteritems() :
            return_str =return_str +'<tr><td>'+name.title() +'</td><td>'+str(maketablefromdict(value))+'</td></tr>'
        return_str = '<table>' + return_str + '</table>'
        return return_str
    elif type(the_dict) is list: 
        for element in the_dict:
            for name,value in element.iteritems() :
                return_str =return_str +'<tr><td>'+name.title()+'</td><td>'+str(maketablefromdict(value))+'</td></tr>'
        return_str = '<table>' + return_str + '</table>'
        return return_str
    else:
        return the_dict

######################################################################
# ROUTE: FLAVOR
######################################################################
def set_default_flavor(name, flavor_names):
    global default_flavor
    default_flavor = name
    selected = {}
    for name in flavor_names:
        selected[name] = ""
    selected[default_flavor] = 'checked'
    return selected
        

def buildFlavorNamesArray(clouds):
    flavor_names=[]
    for name, cloud in clouds.iteritems():
        for id, flavor in cloud['flavors'].iteritems():
            flavor_names.append(flavor['name']);
    return flavor_names;



#@app.route('/flavors/<cloud>/' )
@app.route('/flavors/', methods=['GET','POST'])
def display_flavors(cloud=None):
    radioSelected={}
    flavor_names=buildFlavorNamesArray(clouds.clouds);
    # for debugging
    cloud = 'india-openstack'

    ############reading from yaml file ############
    config_flavor = cm_config()
    activeClouds=config_flavor.active()
    for cloud in activeClouds:
        if 'openstack' in cloud:
            configurations= config_flavor.cloud(cloud)   
            default_flavor=configurations['default']['flavor']
            selected=set_default_flavor(default_flavor, flavor_names)
            radioSelected[cloud]=selected
            print radioSelected
            selected={};
    ############  end of reading from yaml file ############

    time_now = datetime.now().strftime("%Y-%m-%d %H:%M")    
    active = make_active('flavors')
    #selected = set_default_flavor(default_flavor, flavor_names)

    if request.method == 'POST':
        radioSelected={}
        for cloud in activeClouds:
            if 'openstack' in cloud:
                
                default_flavor= request.form[cloud] 
                print default_flavor
                
                ############ writing in yaml file ############
                yamlFile= config_flavor.get();
                yamlFile['clouds'][cloud]['default']['flavor']=default_flavor;
                write_yaml(filename,yamlFile)
                ############ end of writing in yaml file ############
                selected = set_default_flavor(default_flavor, flavor_names)
                radioSelected[cloud]=selected
                print radioSelected
                selected={};
    
      

    if cloud == None:
        pass
    else:
        return render_template('flavor.html',
                               updated=time_now,
                               clouds=clouds.clouds,
                               active=active,
                               version=version,radioSelected=radioSelected)



######################################################################
# ROUTE: CLOUDS
######################################################################

def set_default_clouds(activeClouds, availableClouds):
    selected = {}
    for name in availableClouds:
        selected[name] = ""
        for activeCloud in activeClouds:
            if name in activeCloud:
                selected[name] = 'checked'
    return selected


@app.route('/clouds/', methods=['GET','POST'])
def display_clouds():
    projectSelected={}
    time_now = datetime.now().strftime("%Y-%m-%d %H:%M") 
    active = make_active('clouds')
    config_cloud = cm_config()
    activeClouds=config_cloud.active()
    availableClouds=config_cloud.clouds()
    activeProjects=config_cloud.projects('active')
    selected=set_default_clouds(activeClouds, availableClouds)
    project_names=buildProjectNamesArray(activeProjects)

    for availableCloud in availableClouds:
        projectSelected[availableCloud]=set_default_project("", project_names,'selected');
        for cloud in activeClouds:
            if 'openstack' in cloud:
                configurations= config_cloud.cloud(cloud) 
                default_project=configurations['default']['project']
                projectSelected[cloud]=set_default_project(default_project, project_names,'selected')

    if request.method == 'POST':
        cloudNames = request.form.getlist("clouds")
        yamlFile=config_cloud.get()
        selected=set_default_clouds(cloudNames, availableClouds)
        for cloudName in cloudNames:
            projectName = request.form[cloudName]
            if "None" in projectName:
                projectName=yamlFile['projects']['default']
            yamlFile['clouds'][cloudName]['default']['project']=projectName;
        yamlFile['active']=cloudNames
        write_yaml(filename, yamlFile)
        
        for availableCloud in availableClouds:
            projectSelected[availableCloud]=set_default_project("", project_names,'selected');
            for cloudName in cloudNames:
                if 'openstack' in cloudName:
                    configurations= config_cloud.cloud(cloudName) 
                    default_project=configurations['default']['project']
                    projectSelected[cloudName]=set_default_project(default_project, project_names,'selected')
    
    return render_template('clouds.html',
                               updated=time_now,
                               clouds=availableClouds,
                               active=active,
                               version=version,projects=activeProjects,selected=selected,projectSelected=projectSelected)

######################################################################
# ROUTE: IMAGES
######################################################################

def set_default_image(name, image_names):
    global default_image
    default_image = name
    selected = {}
    for name in image_names:
        selected[name] = ""
    selected[default_image] = 'checked'
   # print default_image;
    return selected
        
default_image = "ktanaka/ubuntu1204-ramdisk.manifest.xml"

def buildImageNamesArray(clouds):
    image_names=[]
    for name, cloud in clouds.iteritems():
        for id, image in cloud['images'].iteritems():
            image_names.append(id);
    return image_names;


#@app.route('/images/<cloud>/')
@app.route('/images/', methods=['GET','POST'])
def display_images():
    radioSelected={}
    time_now = datetime.now().strftime("%Y-%m-%d %H:%M")    
    active = make_active('images')

    image_names=buildImageNamesArray(clouds.clouds);

    ############reading from yaml file ############

    config_image = cm_config()
    activeClouds=config_image.active()
    for cloud in activeClouds:
        if 'openstack' in cloud:
                configurations= config_image.cloud(cloud)   
                default_image=configurations['default']['image']
                selected=set_default_image(default_image, image_names)
                radioSelected[cloud]=selected
                #print radioSelected #this dict will contain which image in whch cloud is checked
                selected={};

     ############  end of reading from yaml file ############

    if request.method == 'POST':
        radioSelected={}
        for cloud in activeClouds:
                if 'openstack' in cloud:
                        
                        default_image= request.form[cloud] 
                        #print default_image

                        ############ writing in yaml file ############
                        yamlFile= config_image.get();
                        yamlFile['clouds'][cloud]['default']['image']=default_image;
                        write_yaml(filename, yamlFile)

                        ############ end of writing in yaml file ############
                        selected = set_default_image(default_image, image_names)
                        radioSelected[cloud]=selected
                        #print radioSelected
                        selected={};

    return render_template('images.html',
                               updated=time_now,
                               clouds=clouds.clouds,
                               active=active,
                               version=version,radioSelected=radioSelected)
    

######################################################################
# ROUTE: TEST 
######################################################################


def set_default_cloud(name, cloud_names):
    global default_cloud
    default_cloud = name
    selected = {}
    for name in cloud_names:
        selected[name] = ""
    selected[default_cloud] = 'checked = ""'
    return selected
        
default_cloud = "india-openstack"


@app.route('/gregor', methods=['GET','POST'])
def gregor():
    global default_cloud
    #    default_cloud = "india-openstack"
    #added by shweta
    config_active = cm_config()
    dict_t = config_active.get('active')
    cloud_names = dict_t;
    #print cloud_names;
    #end of additon by shweta
    #cloud_names = ["india-openstack", "sierra-openstack"] code written by Gregor commented by shweta 
    selected = set_default_cloud(default_cloud, cloud_names)
    
    if request.method == 'POST':
        default_cloud= request.form['selected_cloud']
    #print default_cloud

    selected = set_default_cloud(default_cloud, cloud_names)

    return '''
            <form action="" method="post">
              <input type = "radio"
                 name = "selected_cloud"
                 id = "india-openstack"
                 value = "india-openstack"
                 %(india-openstack)s />
              <label>india-openstack</label>                 
              <input type = "radio"
                 name = "selected_cloud"
                 id = "sierra-openstack"
                 value = "sierra-openstack"
                 %(sierra-openstack)s/>
               <label>sierra-openstack</label>
              <input type=submit value=Update>
           </form>''' %selected


######################################################################
# ROUTE: SECURITY
######################################################################
def set_default_security(name, secGroup_names):
    print secGroup_names
    default_secGroup = name
    selectedSecurity = {}
    for name in secGroup_names:
        selectedSecurity[name] = ""
    selectedSecurity[default_secGroup] = 'checked'
    print selectedSecurity
    return selectedSecurity

@app.route('/security/', methods=['GET','POST'])
def security():
    
     ############reading from yaml file ############
    config_security = cm_config()
    yamlFile=config_security.get()
    securityGroupsList=(yamlFile['security']['security_groups']);
    default_secGroup=yamlFile['security']['default']
    securityGroups=securityGroupsList.keys();
    selectedSecurity=set_default_security(default_secGroup, securityGroups)
     ############  end of reading from yaml file ############
    
    active = make_active('security')
    
    if request.method == 'POST':
        default_secGroup= request.form['selected_securityGroup'] 
        
        ############ writing in yaml file ############
        yamlFile['security']['default']=default_secGroup;
        write_yaml(filename,yamlFile)
        ############ end of writing in yaml file ############
        selectedSecurity=set_default_security(default_secGroup, securityGroups)
    
    
    return redirect("/profile/")

######################################################################
# ROUTE: PROFILE
######################################################################
@app.route('/profile/')
def profile():
        # bug the global var of the ditc should be used
        active = make_active('profile')
        
        config = cm_config()
        activeProjects=config.projects('active')
        dict_t = config.get()
        person = dict_t['profile']
        #print person
        ###########projects radio button################
        activeProjects=config.projects('active')
        project_names=buildProjectNamesArray(activeProjects)
        default_project=dict_t['projects']['default']
        selected=set_default_project(default_project, project_names,'checked')
        ########### end of projects radio button################
        
        ###########security radio button################
        securityGroupsList=(dict_t['security']['security_groups']);
        securityGroups=securityGroupsList.keys();
        default_secGroup=dict_t['security']['default']
        selectedSecurity=set_default_security(default_secGroup, securityGroups)
        ###########end of security radio button################

        selectedClouds = clouds.active()
        defaultClouds = {} # this is wrong, but i just make it so for the mockup, all is contained in cm_config
        
        time_now = datetime.now().strftime("%Y-%m-%d %H:%M")

        address = '\n'.join(str(x) for x in person['address']) 
        return render_template('profile.html',
                               updated=time_now,
                               # keys are also in dict_t, so we may not need that
                               keys="",  # ",".join(clouds.get_keys()),
                               defaultClouds=defaultClouds,
                               selectedClouds=selectedClouds,
                               person=person,
                               address=address,
                               active_clouds=clouds.active(),
                               active=active,
                               config=dict_t,
                               fun_print = lineToFingerprint,
                               selected=selected,
                               version=version,
                               projects=activeProjects,
                               securityGroups=securityGroups,
                               selectedSecurity=selectedSecurity,
                               )

######################################################################
# ROUTE: METRIC
######################################################################
#@app.route('/metric/<s_date>/<e_date>/<user>/<cloud>/<host>/<period>/<metric>')


@app.route('/metric/main', methods=['POST', 'GET'])
def metric():
    global clouds
    args = {"s_date": request.args.get('s_date', ''),
            "e_date": request.args.get('e_date', ''),
            "user": request.args.get('user', ''),
            "cloud": request.args.get('cloud', ''),
            "host": request.args.get('host', ''),
            "period": request.args.get('period', ''),
            "metric": request.args.get('metric', '')}

    return render_template('metric.html',
                           clouds=clouds.get(),
                           metrics=clouds.get_metrics(args),
                           pages=pages,
                           active=make_active('metric'),
                           version=version)

######################################################################
# ROUTE: PAGES
######################################################################


@app.route('/<path:path>/')
def page(path):
    active = make_active(str(path))
    page = pages.get_or_404(path)
    return render_template('page.html',
                           page=page,
                           pages=pages,
                           active=active,
                           version=version)

######################################################################
# ROUTE: KEYS
######################################################################

#
# the key management does not belong in server.py but must be in a  separate class (as do other things in server ....)
# server is done to render things not to implement logig that belongs to cloudmesh.
# keymanagement and checking of keys is a function that should be in cloudmesh. To simplify that, we started a cm_keys code
# 
#
@app.route('/keys/',methods=['GET','POST'])
def managekeys():
    print ">>>>> KEY"
    
    active = make_active('profile')
    active_clouds = clouds.active()
    time_now = datetime.now().strftime("%Y-%m-%d %H:%M")
    
    config = cm_config()
    yamlFile= config.get()    
    haskeys = False
    keydict = {}
    keylist = {}
    defaultkey = ''
    msg = ''
    error = False
    """
    keys:
      default: name 1 
      keylist:
         name 1: file $HOME/.ssh/id_rsa.pub
         name 2: file $HOME/.ssh/id_rsa2.pub
         bla: key ssh-rsa AAAAB3.....zzzz keyname
    """
    if(yamlFile.has_key('keys')):
       haskeys = True 
       keydict = config.userkeys()
       defaultkey = config.userkeys()['default']
       keylist = config.userkeys()['keylist']

    if request.method == 'POST' and request.form.has_key('keyname'):
        keyname = request.form['keyname']
        fileorpath = request.form['keyorpath']
        if not 'ssh-rsa' in fileorpath:
            type = 'file'
        else : 
            type = 'keystring'
        
        if type == "file" :
            fileorpath = os.path.expanduser(fileorpath)
        if keyname is "" or type is 'None' or fileorpath is "":
            error = True
            msg = "Invalid Data. Please Renter Data" 
            
        elif not validateKey(type ,fileorpath): 
            if type.lower() == "file":
                msg = "Invalid file path or Invalid key file" 
            else:
                msg = "Invalid key string"
        elif ' ' in  keyname:
            msg = "Invalid key name, cannot contain spaces"
        elif haskeys and keydict['keylist'].has_key(keyname):
            msg = "Key name already exists"
        else :
            if haskeys :
                keydict['keylist'][keyname] = fileorpath
            else :
                yamlFile['keys'] = {'default':keyname ,'keylist':{keyname: fileorpath}}
                keylist = yamlFile['keys']['keylist']
                defaultkey = yamlFile['keys']['default']
                haskeys = True
            for clud in active_clouds:
                (stat,msg) = clouds.add_key_pair(clud,getKey(fileorpath),keyname)
                print msg
            write_yaml(filename, yamlFile)
            msg = 'Key added successfully'
    elif request.method == 'POST' :
            yamlFile['keys']['default'] = request.form['selectkeys']
            defaultkey = yamlFile['keys']['default']
            write_yaml(filename, yamlFile)
    return render_template('keys.html',
                           haskeys=haskeys,
                           active=active,
                           keylist = keylist,
                           defaultkey =defaultkey,
                           fun_fprint = lineToFingerprint,
                           show = msg)
                        

@app.route('/keys/delete/<name>/')
def deletekey(name):
    active = make_active('profile')

    config = cm_config()
    yamlFile= config.get()
    keydict = yamlFile['keys']
    defaultkey = keydict['default']
    keylist = keydict['keylist']
    if len(keylist) ==1 :
        del yamlFile['keys']
    else :
        del keylist[name]
        if defaultkey == name:
            keydict['default'] = ''
    active_clouds = clouds.active()    
    for clud in active_clouds:
                (stat,msg) = clouds.del_key_pair(clud,name)
                print msg
    write_yaml(filename, yamlFile)
    return redirect("/keys/")

def validateKey(type,file):
    if type.lower() == "file":
        try :
            keystring = open(file, "r").read()
        except :
            return False
    else :
        keystring = file
    
    try :
        type, key_string, comment = keystring.split()
        data = base64.decodestring(key_string)
        int_len = 4
        str_len = struct.unpack('>I', data[:int_len])[0] # this should return 7

        if data[int_len:int_len+str_len] == type:
            return True
    except Exception, e:
        print e
        return False

def getKey(fileorpath):
    if not 'ssh-rsa' in fileorpath:
        fileorpath = os.path.expanduser(fileorpath)
        try :
            keystring = open(fileorpath, "r").read()
            return keystring
        except Exception, e :
            print e
    else : 
        return fileorpath
        


def lineToFingerprint(line,type=None):
    if not 'ssh-rsa' in line:
            type = 'file'
    else : 
            type = 'keystring'
    if type == "file":
        return line
    type,key_string, comment = line.split()
    key = base64.decodestring(key_string)
    fp_plain = hashlib.md5(key).hexdigest()
    return ':'.join(a+b for a,b in zip(fp_plain[::2], fp_plain[1::2]))

def write_yaml(filename, content_dict):
    if with_write:
        d = {}
        d['cloudmesh']=content_dict;
        print "WRITE YAML"
        f = open(filename, "w")
        yaml.safe_dump(d, f, default_flow_style=False, indent=4)
        f.close()

    
if __name__ == "__main__":
    app.run()



import sys
import pymongo
import ConfigParser
from fgmetric.FGConstants import FGConst


class Mongodb:
    def __init__(self):
        # If your database server is running in auth mode, you will need user and
        # database info. Ex:
        #    mongodb_uri = 'mongodb://username:password@localhost:27017/dbname'
        #
        self.load_conf()
        self.mongodb_uri = 'mongodb://%(dbuser)s:%(dbpasswd)s@%(dbhost)s:%(dbport)d/%(dbname)s' % self.get_dbinfo(
        )
        self.db_name = '%(dbname)s' % self.get_dbinfo()

    def connect(self):
        # pymongo.Connection creates a connection directly from the URI, performing
        # authentication using the provided user components if necessary.
        #
        try:
            self.connection = pymongo.Connection(self.mongodb_uri)
            self.database = self.connection[self.db_name]
        except:
            print('Error: Unable to connect to database.')
            self.connection = None

    def insert(self, collection, data):
        # What follows is insert, update, and selection code that can vary widely
        # depending on coding style.
        #
        if self.connection is not None:

            # To begin with, we'll add a few adventurers to the database. Note that
            # nothing is required to create the adventurers collection--it is
            # created automatically when we insert into it. These are simple JSON
            # objects.
            #
            func = getattr(self.database, collection)
            func.insert(data)
            '''{'service': 'openstack',
                                        'cluster': 'india',
                                        'list': { data },
                                        'date': now })
            '''

    def update(self, collection, data):
        if self.connection is not None:
        # Because it seems we forgot to equip Mordo, we'll need to get him
        # ready. Note the dot notation used to address the 'main-hand' key.
        # Don't send a JSON object describing the 'main-hand' key in the
        # context of the 'equipment' key, or MongoDB will overwrite the other
        # keys stored under 'equipment'. Mordo would be embarassed without
        # armor.
        #
        # Note that in python, MongoDB $ operators should be quoted.
        #
        # database.adventurers.update({'name': 'Mordo' },
        #                            {'$set': {'equipment.main-hand': 'staff'}})

        # Now that everyone's ready, we'll send them off through standard
        # output. Unfortunately this adventure is is for adventurers level 10
        # or higher. We pass a JSON object describing our query as the value
        # of the key we'd like to evaluate.
        #
            return

    def find(self, collection, query):
        func = getattr(self.database, collection)
        res = func.find(query)
        return res

    def drop(self, collection):
        # Since this is an example, we'll clean up after ourselves.
        # database.drop_collection('adventurers')
        return

    def load_conf(self):
        self.config_filename = FGConst.DEFAULT_CONFIG_FILENAME
        self.config_filepath = FGConst.DEFAULT_CONFIG_FILEPATH
        self._set_conf()

    def _set_conf(self):
        config = ConfigParser.ConfigParser()
        config.read(self._get_config_file())

        try:
            self.mongodb_information = {
                "dbhost": config.get('MongoDB', 'host'),
                "dbport": int(config.get('MongoDB', 'port')),
                "dbuser": config.get('MongoDB', 'user'),
                "dbpasswd": config.get('MongoDB', 'passwd'),
                "dbname": config.get('MongoDB', 'db')
            }
        except ConfigParser.NoSectionError:
            raise

    def _get_config_file(self):
        return self.config_filepath + "/" + self.config_filename

    def get_dbinfo(self):
        """ Return db information """

        """
        Args:
            n/a
        Return:
            self.mongodb_information (dict)
        """
        return self.mongodb_information



from string import Template
import os

def path_expand(text):
    """ returns a string with expanded variavble """
    template = Template(text)
    result = template.substitute(os.environ)
    return result



import sys
import yaml
from cloudmesh_cloud import cloudmesh_cloud

try:
    from keystoneclient.v2_0 import client
except:
    print "ERROR: keystoneclient is not installed"
    print "       Please see http://docs.openstack.org/developer/python-keystoneclient/"
    sys.exit(1)

class openstack_grizzly_cloud(cloudmesh_cloud):
    _client = client.Client

    def __init__(self, profiledata, defaultproj, projectlist, cloudname):
        self._credentials = None
        self._keystone = None
        cloudmesh_cloud.__init__(self, profiledata, defaultproj, projectlist, cloudname)

    @property
    def keystone(self):
        if self._keystone is None:
            # Create/connect to keystone
            cm_admin = self.admin_data['cm_admin']
            self._keystone = self._client(
                username=cm_admin['OS_USERNAME'],
                password=cm_admin['OS_PASSWORD'],
                tenant_name=cm_admin['OS_TENANT_NAME'],
                auth_url=cm_admin['OS_AUTH_URL'],
                cacert=cm_admin['OS_CACERT']
                )
        return self._keystone

    def get_user_by_name(self, name):
        user = [u for u in self.keystone.users.list() if u.name == name]
        return user[0] if len(user) == 1 else None

    def get_role_by_name(self, name):
        role = [r for r in self.keystone.roles.list() if r.name == name]
        return role[0] if len(role) == 1 else None

    def get_tenant_by_name(self, name):
        tenant = [t for t in self.keystone.tenants.list() if t.name == name]
        return tenant[0] if len(tenant) == 1 else None

    def initialize_cloud_user(self):
        creds = self.credentials
        defaults = self.clouddefaults
        password = self.newpass()

        creds['OS_USERNAME'] = self.username
        creds['OS_PASSWORD'] = password
        defaults['project'] = self.defaultproject

        # Create user or reset password (we cannot retrieve existing password)
        user = self.get_user_by_name(self.username)
        if user is None:
            user = self.keystone.users.create(self.username, password, self.email)
        else:
            self.keystone.users.update_password(user, password)

        # Create membership role for user in each tenant
        member_role = self.get_role_by_name('_member_')
        for tname in self.projects:
            tenant = self.get_tenant_by_name(tname)
            if tenant is None:
                tenant = self.keystone.tenants.create(tname)
            self.keystone.roles.add_user_role(user, member_role, tenant)
            os_tenant = '%s_OS_TENANT_NAME' % tname.upper()
            creds[os_tenant] = tname
        creds['OS_TENANT_NAME'] = '$%s_OS_TENANT_NAME' % self.defaultproject.upper()




class mock_cloud:
    """ Provides a mock class for testing.  Represents the cloud
    credential data that will come from cloud API"""

    def __init__(self, profiledata, defaultproj, projectlist, cloudname):
        self._username = profiledata['username']
        self._cloudname = cloudname
        self._data = None

    def _load_data(self):
        if self.cloudname == 'india-openstack':
            self._data = {
                'cm_label': 'ios',
                'cm_host': 'india.futuregrid.org',
                'cm_type': 'openstack',
                'credentials': {
                    'OS_AUTH_URL': 'url',
                    'OS_PASSWORD': 'password',
                    'OS_TENANT_NAME': 'member',
                    'OS_USERNAME': self._username,
                    'OS_VERSION': 'essex',
                    'OS_CACERT': '$HOME/.futuregrid/india/openstack/cacert.pem'
                    },
                'default': {
                    'flavor': 'm1.tiny',
                    'image': 'ktanaka/ubuntu1204-ramdisk.manifest.xml',
                    'project': 'fg-181'
                    }
                }
        elif self.cloudname == 'sierra-openstack-grizzly':
            self._data = {
                'cm_label': 'ios',
                'cm_host': 'abc.futuregrid.org',
                'cm_type': 'openstack',
                'credentials': {
                    'OS_AUTH_URL': 'url',
                    'OS_PASSWORD': 'password',
                    'OS_TENANT_NAME': 'member',
                    'OS_USERNAME': self._username,
                    'OS_VERSION': 'grizzly',
                    'OS_CACERT': '$HOME/.futuregrid/india/openstack/cacert.pem'
                    },
                'default': {
                    'flavor': 'm1.tiny',
                    'image': 'ktanaka/ubuntu1204-ramdisk.manifest.xml',
                    'project': 'fg-181'
                    }
                }

    def initialize_cloud_user(self):
        pass

    @property
    def cloudname(self):
        return self._cloudname

    @property
    def data(self):
        if self._data is None:
            self._load_data()
        return self._data





























import ldap

class ldap_user:
    """ Provides a class for getting the user/project
    data that will come from LDAP. """

    # put these in a config file?
    LDAP_HOST = 'im3r.idp.iu.futuregrid.org'
    LDAP_PERSONBASE = 'ou=People,dc=futuregrid,dc=org'
    LDAP_PROJECTBASE = 'ou=Groups,dc=futuregrid,dc=org'

    def __init__(self, username):
        self._username = username
        self._data = None

    def _load_data(self):
        # specify ca cert (make sure TLS_CACERTDIR is defined in
        # /etc/openldap/ldap.conf, or optionally set LDAPTLS_CACERTDIR
        # in environment)
        ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_DEMAND)

        # connect
        ldapconn = ldap.initialize('ldap://%s' % self.LDAP_HOST)

        # start tls connection and bind
        ldapconn.start_tls_s()
        ldapconn.simple_bind_s()

        # Get profile and project data from LDAP
        self._data = {}

        search_result_person = ldapconn.search_s(self.LDAP_PERSONBASE, ldap.SCOPE_SUBTREE, '(cn=%s)' % self.username)
        self._data['person'] = search_result_person[0][1] if search_result_person is not None else None

        self._data['projects'] = ldapconn.search_s(self.LDAP_PROJECTBASE, ldap.SCOPE_SUBTREE, '(&(memberUid=%s)(cn=fg*))' % self.username, ['cn'])


    @property
    def username(self):
        return self._username

    @property
    def data(self):
        if self._data is None:
            self._load_data()
        return self._data

    @property
    def uid(self):
        return self.data['person']['uidNumber'][0] if 'uidNumber' in self.data['person'] else None

    @property
    def gid(self):
        return self.data['person']['gidNumber'][0] if 'gidNumber' in self.data['person'] else None

    @property
    def firstname(self):
        return self.data['person']['givenName'][0] if 'givenName' in self.data['person'] else None

    @property
    def lastname(self):
        return self.data['person']['sn'][0] if 'sn' in self.data['person'] else None

    @property
    def phone(self):
        return self.data['person']['telephoneNumber'][0] if 'telephoneNumber' in self.data['person'] else None

    @property
    def email(self):
        return self.data['person']['mail'][0] if 'mail' in self.data['person'] else None

    @property
    def address(self):
        return self.data['person']['homePostalAddress'] if 'homePostalAddress' in self.data['person'] else None

    @property
    def keys(self):
        keys = {}
        if 'sshPublicKey' in self.data['person']:
            for sshkey in self.data['person']['sshPublicKey']:
                (keytype, key, nickname) = sshkey.split()
                keys[nickname] = "key %s" % sshkey
            return keys
        else:
            return None

    @property
    def activeclouds(self):
        """Not yet implemented"""
        return ['sierra-openstack-grizzly']

    @property
    def defaultcloud(self):
        """Not yet implemented"""
        return 'sierra-openstack-grizzly'

    @property
    def activeprojects(self):
        """List of active projects"""
        if self.data['projects']:
            projects = []
            for project in self.data['projects']:
                (projectDN, projectAttrs) = project
                projectCN = projectAttrs['cn'][0]
                projects.append("%s" % projectCN)
        return projects

    @property
    def completedprojects(self):
        """Not yet implemented"""
        return []

    @property
    def defaultproject(self):
        """The user's default project (currently just the first active project)"""
        return self.activeprojects[0]


if __name__ == "__main__":
    luser = ldap_user('astreib')
    print "Username: %s" % luser.username
    print "Uid: %s" % luser.uid
    print "Gid: %s" % luser.gid
    print "First Name: %s" % luser.firstname
    print "Last Name: %s" % luser.lastname
    print "Keys: %s" % luser.keys
    print "Phone: %s" % luser.phone
    print "Email: %s" % luser.email
    print "Address: %s" % luser.address
    print "Projects - Active: %s" % luser.activeprojects
    print "Projects - Completed: %s" % luser.completedprojects
    print "Projects - Default: %s" % luser.defaultproject
    print "Active Clouds: %s" % luser.activeclouds
    print "Default Cloud: %s" % luser.defaultcloud



from cloudmesh.mongodb import Mongodb


def example():
    cm_mongodb = Mongodb()
    cm_mongodb.connect()
    res = {}  # dict data structure from cloud mesh
    cm_mongodb.insert("cloud_mesh", res)



class cm_projects:

    def __init__(self):
        """initializes based on cm_config and returns pointer to the keys dict."""
        
    def default(self, name)
        """sets the default project"""
        
    def add(self, name, status="active")
        """adds a project with given status"""

    def names(self, status="active"):
        """returns all projects in an array whi it is in a specified status"""

    def validate(self, line):
        """validates if a default project is set"""

    def __str__(self):
        """returns the dict in a string representing the project"""

    def update(self):
        """writes the updated dict to the config"""



""" run with

nosetests -v --nocapture

or

nosetests -v

"""
import sys
#from cloudmesh.openstack.cm_table import table as cm_table
#from cloudmesh.cm_config import cm_config
#from cloudmesh.openstack.cm_compute import openstack
from cloudmesh import cloudmesh

import json
import pprint
pp = pprint.PrettyPrinter(indent=4)

header = True

def HEADING(txt):
    if True:
        print
        print "#", 70 * '#'
        print "#", txt
        print "#", 70 * '#'


class Test_cloudmesh:

    def setup(self):
        self.c = cloudmesh()
        self.c.config()
        print self.c

    def tearDown(self):
        pass
        #print self.c.clouds

    def test_001_print(self):
        HEADING("test_001_refresh")
        print self.c



    def test_101_refresh(self):
        HEADING("test_101_refresh")
        self.c.refresh("india-openstack")
        self.c.dump()

    def test_103_refresh_all(self):
        HEADING("test_103_refresh_all")
        print self.c.refresh()
         
    def test_102_str(self):
        HEADING("test_102_str")
        print self.c
         
    def test_103_str(self):
        HEADING("test_103_str")
        print self.c.dump()

    def test_103_info(self):
        HEADING("test_103_info")
        self.c.refresh()
        self.c.info()


    




#! /usr/bin/env python

from datetime import datetime
from fabric.api import *
from random import randint
import ConfigParser
import pickle
import os 
import sys
import json

import commands
import getpass

from sh import azure as _azure
from sh import fgrep as _fgrep
import pprint


from multiprocessing import Pool as _Pool
from cloudmesh.cm_config import cm_config
global maxparallel;


maxparallel = 5


######################################################################
# Baked sh azure functions to make azure client easier to use
######################################################################
vm_start = _azure.bake("--json","vm","start")
vm_restart = _azure.bake("--json","vm","restart")
vm_shutdown = _azure.bake("--json","vm","shutdown")
vm_list = _azure.bake("--json","vm","list")
vm_create = _azure.bake("--json","vm","create")
vm_show = _azure.bake("--json","vm","show")
vm_delete = _azure.bake("--json","vm","delete")
vm_image_list = _azure.bake("--json","vm","image", "list")

account = _azure.bake("account")

######################################################################
# azure class
######################################################################

class cm_azure:
    images = {}
    servers = {}
    cloud = None
    label = ""
    flavors = {}
    credentials = {}
    type = "azure"   # global var
    
    filename = "%(home)s/%(location)s" % {
        "home" : os.environ['HOME'], 
        "location" : ".futuregrid/cloudmesh.yaml"
        }

    

    ######################################################################
    # initialize
    ######################################################################

    def __init__(self,
                 username=None,
                 password=None,
                 publish_settings_file_path=None,
                 default_image_name=None):
        """
        initializes the openstack cloud from a defould novaRC file
        locates at ~/.futuregrid.org/openstack. However if the
        parameters are provided it will instead use them
        """

        self.clear()
        self.config (username,
                password,
                publish_settings_file_path,
                default_image_name)
        

    def config (self,username=None,
                password=None,
                publish_settings_file_path=None,
                default_image_name=None):

        if username == None:
            config = cm_config()
            configuration = config.get('azure')
            self.credentials['username'] = configuration['username']
            self.credentials['password'] = configuration['password']
            self.credentials['settings']= {'publishsettings_file_path' : configuration['publishsettings_file_path'],
                                            'defaultimage' : configuration['defaultimage']}
        else :
            self.credentials['username'] = username
            self.credentials['password'] = password
            self.credentials['settings']= {'publishsettings_file_path' : publish_settings_file_path,
                                            'defaultimage' : default_image_name}
        return

    

    ######################################################################
    # get methods
    ######################################################################

    def get(self):
        """returns the dict with all the sms"""
        return self.servers

    ######################################################################
    # print methods
    ######################################################################

    def __str__ (self):
        """
        print everything but the credentials that is known about this
        cloud in json format.
        """
        information = {
            'label': self.label,
            'flavors': self.flavors,
            'vms': self.servers,
            'images' : self.images}
        return json.dumps(information, indent=4)

    def type():
        return self.type

    ######################################################################
    # refresh 
    ######################################################################
    
    def refresh(self, type=None):

        time_stamp = datetime.now().strftime('%Y-%m-%dT%H-%M-%SZ')
        selection = ""
        if type:
            selection = type.lower()[0]
            all = selection == 'a'
        else:
            all = True

        if all:
            self.refresh('images')
            self.refresh('vms')
            self.refresh('flavors')
            return

        key = ""
        list_function = vm_list
        if selection == 'i' :
            key = 'Name'
            list_function = vm_image_list
            store = self.images
        elif selection == 'v' :
            key = 'VMName'
            list_function = vm_list
            store = self.servers
        
        if selection != 'f' :
            list = json.loads(str(list_function()))
            for object in list:
                id = object[key]
                store[id] = object
                store[id]['cm_refresh'] = time_stamp


       
        if selection == 'f' :
            # TODO: move to YAML fileas "flavors : { ...} under azure, copy from there and deal only with timestamp here
            # once in local dict at config time , no reason to reread this portion of yaml file
            self.flavors = {
                'ExtraSmall' : {
                    'CPU_Cores': 'Shared',
                    'Memory':'768 MB',
                    'Disk_Space_for_Web_&_Worker_Roles':'19,480 MB',
                    'Disk Spacein a VM Role':'20 GB',
                    'Bandwidth' :'5 Mbps',
                    'cm_refresh': time_stamp
                },
                'Small' : {
                    'CPU_Cores': '1',
                    'Memory':'1.75 GB',
                    'Disk_Space_for_Web_&_Worker_Roles':'229,400 MB',
                    'Disk Spacein a VM Role':'165 GB',
                    'Bandwidth' :'100 Mbps',
                    'cm_refresh': time_stamp
                },
                'Medium' : {
                    'CPU_Cores': '2',
                    'Memory':'3.5 GB',
                    'Disk_Space_for_Web_&_Worker_Roles':'500,760 MB',
                    'Disk Spacein a VM Role':'340 GB','Bandwidth' :'200 Mbps',
                    'cm_refresh': time_stamp
                },
                'Large' : {
                    'CPU_Cores': '4',
                    'Memory':'7 GB',
                    'Disk_Space_for_Web_&_Worker_Roles':'1,023,000 MB',
                    'Disk Spacein a VM Role':'850 GB',
                    'Bandwidth' :'400 Mbps',
                    'cm_refresh': time_stamp
                },
                'ExtraLarge' : {
                    'CPU_Cores': '8', 'Memory':'14 GB',
                    'Disk_Space_for_Web_&_Worker_Roles':'2,087,960 MB',
                    'Disk Spacein a VM Role':'1890 GB','Bandwidth' :'800 Mbps',
                    'cm_refresh': time_stamp
                }
            }
        


    ######################################################################
    # manage vms 
    ######################################################################

    def start(self,name):
        """starts a vm with the given name"""
        result = vm_start(name)
        # add result to internal cache
        print result

    def restart(self,name):
        """restarts a vm with the given name"""
        result = vm_restart(name)
        # add result to internal cache
        print result

    def shutdown(self,name):
        """shutdown of a vm with the given name"""
        result = vm_shutdown(name)
        # add result to internal cache
        print result


    def list(self):
            result = vm_list()
            # add result to internal cache
            print result

    def show(self,name):
            result = vm_show()
            # add result to internal cache
            print result

    def vm_delete(self,name):
            result = vm_delete(name)
            # add result to internal cache
            return result

    def vms_delete(self,names):
           
        for name  in names:
            print "Deleting %s" % name
            result = vm_delete(name)
            # add result to internal cache
        return names

    def vms_find(self):
        ids = []
        for (id, vm)  in self.servers.items():
                ids.append(id)

        return ids
    
    def vm_create(self,name=None,image_name=None, flavor_name=None):
        #cmd = 'azure vm create %(vmname)s %(image)s %(username)s --ssh --location "East US" %(password)s' % vm
        #print cmd

        #
        # there may also be parameter to start up multiple vms find out and integrate with np=None, if None than np=1
        #
        
        if flavor_name== None :
            result = vm_create("%s" % self._vm_name(randint(0,1000)) if name == None else name,
                               "%(defaultimage)s" % self.credentials['settings'] if image_name == None else image_name,
                               "%(username)s"% self.credentials,
                               "--ssh",
                               "--location",
                               "East US",
                               "%(password)s" % self.credentials,
                               )
        else :
            result = vm_create("%s" % self._vm_name(randint(0,1000)) if name == None else name,
                               "%(defaultimage)s" % self.credentials['settings'] if image_name == None else image_name,
                               "%(username)s"% self.credentials,
                               "--ssh",
                               "--location",
                               "East US",
                               "%(password)s" % self.credentials,
                               "-z",
                               flavor_name
                               )
                            
        return result
    

    def _vm_name(self,index):
        number = str(index).zfill(3)
        name = '%s-%s' % (self.credentials['username'], number)
        return name

    ######################################################################
    # activate azure
    ######################################################################

    def activate(self):

        result = account('clear')
        print result

        errmsg = 'No account information found' 
        cmd = 'azure account import \'%(publishsettings_file_path)s\'' % self.credentials['settings'];

        text =  commands.getstatusoutput(cmd)
        if not errmsg in text[1] :
            print 'Activated'
        else :
            print 'There was an error while activation'

    ######################################################################
    # functions we want to rewrite/replace
    ######################################################################

    def _selectImage() :
        # I AHVE NO IDEA WHAT THIS IS DUE TO NON INFORMATIVE VARIABLE NAMEING
        images = _buildAzureImageDict()
        print 'Please select Image'    ;

        # what is arg1, arg2?
        for (arg1, arg2) in images.items() :
            print arg1 + "\t" + arg2[0] +"\t" +arg2[1] + "\t" +arg2[2]

        while 1 :
            var = raw_input("Image : ")
            if images.has_key(var) :
                print images[var][0]
                return images[var][0]
            else :
                print "Incorrect Image name"
        return var



    def _copy_image_list_to_dict(self) :
        """
        method not needes as so simple, instead work on refresh from
        openstack
        """
        images = vm_list()

        for key in vms :
            image = image[key]
            self.images[image['Name']] = image
            #see other attributes in openstack refersh class
        return
    

    def clear(self):
        """
        clears the data of this azure instance, a new connection
        including reading the credentials and a refresh needs to be
        called to obtain again data.
        """
        self.type = "azure"
        self.flavors = {}
        self.images = {}
        self.servers = {}
        self.credential = {}
        self.cloud = None


if  __name__ =='__main__':
    
    cloud = cm_azure()
    cloud.refresh("all")
    print cloud.vms_find();
    #cloud.refresh("vms")
    #cloud.refresh("images")
    #print cloud.vm_create('ppnewaskar-228','b39f27a8b8c64d52b05eac6a62ebad85__Ubuntu-12_04_1-LTS-amd64-server-20121218-en-us-30GB',
    #'large')
    #print cloud
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        




class table:
    '''
    format = HTML | ASCII | "%20s"
    type is not used and wil be in future removed
    columns = number of columns
    structure is an array of rows
    header = True the firts row is a headline
    '''
    _content = ""

    def __init__(self):
        self._content = ""
        return

    def __str__(self):
        return self._content

    def create(self, structure, columns, format=None, type=None, header=True):
        self._content = self._begin(format)

        # Create header
        line = ""
        for col in columns:
            line += str(self._cell(str(col), format))
        self._content += self._row(line, format)

        # Create Rows
        for element in structure.items():
            attributes = element[1]
            line = ""
            for col in columns:
                line += self._cell(str(attributes[col]), format)
            self._content += self._row(line, format)
        self._content += self._end(format)

    def _begin(self, format=None):
        if format == 'HTML':
            return '<table>\n'
        return ""

    def _end(self, format=None):
        if format == 'HTML':
            return str('</table>\n')
        return ""

    def _cell(self, value, format=None):
        if format == None or format == 'ASCII':
            return value + ' '
        if format.startswith('%'):
            field = format % value
            return field
        if format == 'HTML':
            return '<td>' + value + '</td>'
        return ""

    def _row(self, value, format=None):
        if format == None or format == 'ASCII' or format.startswith('%'):
            return value + '\n'
        if format == 'HTML':
            return '    <tr>' + value + '</tr>\n'
        return ""



#! /usr/bin/env pythos
from pprint import pprint
import json
import os
import sys
from sh import fgrep
import novaclient

from novaclient.v1_1 import client


class credentials:

    location = '.futuregrid/openstack/novarc'

    username = None
    password = None
    project = None
    label = None
    url = None
    home = os.environ['HOME']

    filename = "%s/%s" % (home, location)

    variables = {
        "project": "OS_TENANT_NAME",
        "username": "OS_USERNAME",
        "password": "OS_PASSWORD",
        "url": "OS_AUTH_URL"
    }

    def type(self, name):
        if name == 'openstack':
            self.variables = {
                "project": "OS_TENANT_NAME",
                "username": "OS_USERNAME",
                "password": "OS_PASSWORD",
                "url": "OS_AUTH_URL"
            }
        if name == 'eucalyptus':
            self.variables = {
                "accesskey": "EC2_ACCESS_KEY",
                "secretkey": "EC2_SECRET_KEY"
            }

    def __init__(self, label, filename=None):
        if filename != None:
            self.filename = filename
        self._load()
        self.label = label
        return

    def _get_rc_variable(self, variable):
        (attribute, value) = fgrep("export " + self.variables[
                                   variable], self.filename).replace("\n", "").split("=")
        return value

    def _load(self):

        self.project = self._get_rc_variable('project')
        self.username = self._get_rc_variable('username')
        self.password = self._get_rc_variable('password')
        self.url = self._get_rc_variable('url')

        return credentials

    def __str__(self):
        return json.dumps(
            {'username': self.username,
             'password': self.password,
             'url': self.url,
             'project': self.project},
            indent=4)

    def set(name, value):
        self.credentials.name = value

if __name__ == "__main__":

    credential = credentials('india')

    print credential

    """
    flavors = nova.flavors.list()
    for flavor in flavors:
        details = flavor.__dict__
        pprint(details)
    """



#! /usr/bin/env python
"""cm config
------------
Command to generate rc files from our cloudmesh configuration files.

Usage:
  cm defaults [FILENAME] - stores also alias
  cm defaults clear - clears the defaults
  
  cm set PREFIX INDEX - automatic index index must be of the form 00001  from which to start naming vms. 0 indicate how many leading 0s in the index.
  cm set project [PROJECT] - set functions will be interactive if no param is given
  cm set cloud [CLOUD]
  cm set image [IMAGE] [CLOUD]
  cm set flavor [FLAVOR] [CLOUD]

  cm list [CLOUD | all] 
  cm list servers [CLOUD]
  cm list images [CLOUD]
  cm list flavors [CLOUD]
  cm list config [CLOUD]

  cm find REGEX - finds the fm that match the condition  (cloud="name", ...)

  cm start COUNT [IMAGE] [FLAVOR] [CLOUD]
  cm stop [INDEX] 
  cm delete [INDEX] 

  cm info [CLOUD]
  cm info [NAME] - info of the vom with name
  cm info [INDEX] - info about the vm with the given index

  cm referesh [all | CLOUD] [images | servers | flavors] 
 
  cm alias ALIAS=STRING

  cm rain IMAGE CLOUD

  cm usage START END [CLOUD] [PROJECT]
  
  cm --version
  cm --help

  
  cm config ... take from cm_cli

"""
from docopt import docopt


if __name__ == '__main__':

    print "not yet implemented"



import pickle
from sh import fgrep
from sh import nova
from sh import tail
from datetime import datetime
import json
import sys
import os
import pprint
pp = pprint.PrettyPrinter(indent=4)

# import shelve
from cm_config import cm_config
#from openstack.cm_compute import openstack as os_client

#Error Cannot Import Openstack
from openstack.cm_compute import openstack

from eucalyptus.eucalyptus_new import eucalyptus
try:
    from azure.cm_azure import cm_azure as azure 
except:
    print "AZURE NOT ENABLED"

class cloudmesh:

    ######################################################################
    # global variables that define the information managed by this class
    ######################################################################

    datastore = "data/clouds.txt"

    # dict that holds vms, flavors, images for al iaas
    clouds = {}

    # array with keys from the user
    keys = []

    configuration = {}
    
    ######################################################################
    # initialization methods
    ######################################################################

    def __init__(self):
        self.clear()
        #Read Yaml File to find all the cloud configurations present
        self.config()

    def clear(self):
        self.clouds = {}
        self.keys = []

    ######################################################################
    # the configuration method that must be called to get the cloud info
    ######################################################################

    def config(self):
        """
        reads the cloudmesh yaml file that defines which clouds build
        the cloudmesh
        """

        print "CONFIG"
        self.configuration = cm_config()


        #pp.pprint (configuration)


        active_clouds = self.configuration.active()
        print active_clouds
        
        for cloud_name in active_clouds:
            try:
                credential = self.configuration.get(cloud_name)
                cloud_type = self.configuration.get()['clouds'][cloud_name]['cm_type']

                if cloud_type in ['openstack','eucalyptus','azure']:
                    self.clouds[cloud_name] = {'name': cloud_name,
                                               'cm_type': cloud_type,
                                               'credential': credential}
            except: #ignore
                pass

        return

    ######################################################################
    # importnat get methods
    ######################################################################

    def get(self):
        """returns the dict that contains all the information"""
        return self.clouds

    def active(self):
        active_clouds = self.configuration.active()
        return active_clouds

    def prefix(self):
        return self.configuration.prefix()

    def profile(self):
        return self.configuration.profile()

    def default(self,cloudname):
        return self.configuration.default(cloudname)

    ######################################################################
    # important print methods
    ######################################################################

    def __str__(self):
        return str(self.clouds)
    
    def dump(self):
        print json.dumps(self.clouds, indent=4)


    ######################################################################
    # find 
    ######################################################################        
    #
    # returns dicts of a particular type
    #
    ######################################################################    

    def find (cloud=["all"], type="servers", project=["all"]):
        """
        Returns a dict with matching elements

        cloud = specifies an array of cloud names that are defined in
        our configuration and returns matching elements in a dict. The
        first level in the dict is the cloud, the second level are the
        elements. If all is specified we serach in all clouds

        type = "servers", "images", "flavors"

        The type specifies the kind of element that we look for
        (we only look for the first character e.g. s, i, f)

        project = an array of projects we search for. This only
        applies for servers for now. Till we have introduced the
        profile that restricts available images and flavors for a
        project.

        In all cases None can be used as an alternative to ["all"]
        
        """
        result = {}
        return result

    ######################################################################
    # the refresh method that gets upto date information for cloudmesh
    # If cloudname is provided only that cloud will be refreshed 
    # else all the clouds will be refreshed
    ######################################################################

    def cloud_provider (self, type):
        provider = None
        if type == 'openstack':
            provider = openstack
        elif type == 'eucalyptus':
            provider = eucalyptus
        elif type == 'azure':
            provider = azure
        return provider

    def info(self):
        print 70 * "="
        print "CLOUD MESH INFO"
        print 70 * "="
        try:
            for name in self.clouds.keys():
                cloud_type = self.clouds[name]['cm_type']
                provider = self.cloud_provider(cloud_type)
                cloud = provider(name)
                print
                print "Info", name
                print 70 * "-"
                cloud.refresh("all")
                cloud.info()
                
        except Exception, e:
            print e
        



    def refresh(self, names=["all"], types=["all"]):
        """
        This method obtians information about servers, images, and
        flavours that build the cloudmesh. The information is held
        internally after a refresh. Than the find method can be used
        to query form this information. To pull new information into
        this data structure a new refresh needs to be called.

        Usage is defined through arrays that are passed along.

        type = "servers", "images", "flavors"

        The type specifies the kind of element that we look for
        (we only look for the first character e.g. s, i, f)

        In all cases None can be used as an alternative to ["all"]
        
        if cloud name  == None and type = none update everything

        if cloud name !=None and type = none update everything in the
        specified clouds

        if cloud name != None and type != none
           refresh the given types for the given clouds
        
        """
        if types == ['all'] or type == None:
            types = ['servers','flavors','images']

        if names == ['all'] or names == None:
            names = self.clouds.keys()
            
        # at one point use a threadpool.
        try:
            for name in names:
                cloud_type = self.clouds[name]['cm_type']
                provider = self.cloud_provider(cloud_type)
                cloud = provider(name)
                print "Refresh cloud", name
                for type in types:
                    print "    Refresh ", type
                    cloud.refresh(type=type)
                    result = cloud.get(type)
                    self.clouds[name][type] = cloud.get(type)
                    #maye be need to use dict update ...
                    self.clouds[name].update({'name': name, 'cm_type': cloud_type})
                
        except Exception, e:
            print e
    
    def refresh_user_id(self, names=["all"]):
        if names == ['all'] or names == None:
            names = self.clouds.keys()        
        try:
            for name in names:
                cloud_type = self.clouds[name]['cm_type']
                provider = self.cloud_provider(cloud_type)
                cloud = provider(name)
                if not self.clouds[name].has_key('user_id'):
                    self.clouds[name]['user_id'] = cloud.find_user_id()
        except Exception, e:
            print e
    
    def add(self, name, type):
        try:
            self.clouds[name]
            print "Error: Cloud %s already exists" % name
        except:
            self.refresh(name, type)

    """
    def get_keys(self):
        return self.keys

    def refresh_keys(self):
        self.keys = []
        result = fgrep(tail(nova("keypair-list"), "-n", "+4"),"-v","+")
        for line in result:
            (front, name, signature, back) = line.split("|")
            self.keys.append(name.strip())
        return self.keys


    def refresh(self):
        keys = self.refresh_keys()
        for cloud in keys:
            self.refresh(cloud)

        # p = Pool(4)
        # update = self.refresh
        # output = p.map(update, keys)

    """

    ######################################################################
    # saves and reads the dict to and from a file
    ######################################################################
    def save(self):
        #tmp = self._sanitize()
        file = open(self.datastore, 'wb')
        # pickle.dump(self.keys, file)
        pickle.dump(tmp, file)
        file.close()

    def load(self):
        file = open(self.datastore, 'rb')
        # self.keys = pickle.load(file)
        self.clouds = pickle.load(file)
        ''' above returns:
        [u'gvonlasz']
         So, call pickle again to get more:
            {'india': {'name': 'india',
            'servers': {u'2731c421-d985-44ce-91bf-2a89ce4ba033': {'cloud': 'india',
            'id': u'2731c421-d985-44ce-91bf-2a89ce4ba033',
            'ip': u'vlan102=10.1.2.85, 149.165.158.7',
            'name': u'gvonlasz-001',
            'refresh': '2013-02-11 20:30:04.472583',
            'status': u'ACTIVE'},
            ...
        '''
        self.clouds = pickle.load(file)
        file.close()

    ######################################################################
    # TODO: convenient +, += functions to add dicts with cm_type
    ######################################################################

    def delete(self, cloud_name, server_id):
        try:
            cloud_type = self.clouds[cloud_name]['cm_type']
            provider = self.cloud_provider(cloud_type)
            cloud = provider(cloud_name)
            cloud.vm_delete(server_id) 
        except:
            print "Error: could not delete", cloud_name, server_id


    def add_key_pair(self,cloud_name,key,name):
        try:
            cloud_type = self.clouds[cloud_name]['cm_type']
            provider = self.cloud_provider(cloud_type)
            cloud = provider(cloud_name)
            return cloud.upload_key_pair(key,name) 
        except:
            print "Error: could not update keypair", cloud_name
    def del_key_pair(self,cloud_name,name):
        try:
            cloud_type = self.clouds[cloud_name]['cm_type']
            provider = self.cloud_provider(cloud_type)
            cloud = provider(cloud_name)
            return cloud.delete_key(name) 
        except:
            print "Error: could not delete keypair", cloud_name

    def assign_public_ip(self,cloud_name,vm_id):
        cloud_type = self.clouds[cloud_name]['cm_type']
        # for now its only for openstack 
        if cloud_type in "openstack" :
            provider = self.cloud_provider(cloud_type)
            cloud = provider(cloud_name)
            cloud.assign_public_ip(vm_id, cloud.get_public_ip().ip)
        # code review: GVL
        # else:
        #    print "BUG: assigning ip addresses from other clouds such as azure, and eucalyptus not implemented yet." 
            
        
    def create(self, cloud_name, prefix, index, image_id, flavor_name, key= None, security_group=None):
        
        print ">>>>>>",  cloud_name, prefix, index, image_id, flavor_name, key
        
        security_groups=[]
        name = prefix + "-" + index
        
        cloud_type = self.clouds[cloud_name]['cm_type']
        print cloud_type
        provider = self.cloud_provider(cloud_type)

        cloud = provider(cloud_name)

        if security_group == None:
            security_group=cloud.checkSecurityGroups() 
        
        if  not security_group == None:
            security_groups.append(security_group)
        else :
            security_groups = None
        return cloud.vm_create(name, flavor_name, image_id , security_groups, key)

        """
        keyname = ''
        try:
            cloud_type = self.clouds[cloud_name]['cm_type']
            
            if cloud_type in "openstack":
                config = cm_config()
                yamlFile= config.get()
                if yamlFile[cloud_name]['cm_type'] in 'openstack':
                    if yamlFile[cloud_name].has_key('keypair') :
                        keyname = yamlFile[cloud_name]['keypair']['keyname']

            provider = self.cloud_provider(cloud_type)
            cloud = provider(cloud_name)
            if cloud_name == "india-openstack":
                flavor_name = "m1.tiny"  # this is a dummy and must be retrieved from flask
                image_id = "6d2bca76-8fff-4d57-9f29-50378539b4fa"
                name = prefix + "-" + index
                if (len(keyname) > 0) :
                    cloud.vm_create(name, flavor_name, image_id,keyname )
                else :
                    cloud.vm_create(name, flavor_name, image_id)
                # this is a dummy and must be retrieved from flask
        except Exception , e:
            print "Error: could not create", cloud_name, prefix, index, image_id, e
        """

    ######################################################################
    # TODO: convenient +, += functions to add dicts with cm_type
    ######################################################################

    def __add__(self, other):
        """
        type based add function c = cloudmesh(...); b = c + other
        other can be a dict that contains information about the object
        and it will be nicely inserted into the overall cloudmesh dict
        the type will be identified via a cm_type attribute in the
        dict Nn attribute cm_cloud identifies in which cloud the
        element is stored.
        """
        if other.cm_type == "image":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "vm":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "flavor":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "cloudmesh":
            print "TODO: not implemented yet"
            return
        else:
            print "Error: %s type does not exist", cm_type
            print "Error: Ignoring add"
            return

    def __iadd__(self, other):
        """
        type based add function c = cloudmesh(...); c += other other
        can be a dict that contains information about the object and
        it will be nicely inserted into the overall cloudmesh dict the
        type will be identified via a cm_type attribute in the dict.
        Nn attribute cm_cloud identifies in which cloud the element is
        stored.
        """
        if other.cm_type == "image":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "vm":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "flavor":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "cloudmesh":
            print "TODO: not implemented yet"
            return
        else:
            print "Error: %s type does not exist", cm_type
            print "Error: Ignoring add"
            return

##########################################################################
# MAIN METHOD FOR TESTING
##########################################################################

if __name__ == "__main__":

    c = cloudmesh()
    print c.clouds
    """
    c.config()

    c.dump()


    c = cloud_mesh()

    c.refresh()
    c.add('india', 'openstack')
    c.add('sierra', 'openstack')
    c.refresh_keys()
    c.dump()
    c.save()
    print 70 * "-"
    c.clear()
    c.dump()
    print 70 * "-"
    c.load()
    c.dump()
    print 70 * "-"
    """

    """
    india_os = {
        "OS_TENANT_NAME" : '',
        "OS_USERNAME" : '',
        "OS_PASSWORD" : '',
        "OS_AUTH_URL" : '',
        }


    (attribute, passwd) = fgrep("OS_PASSWORD","%s/.futuregrid/openstack/novarc" % os.environ['HOME']).replace("\n","").split("=")

    india_os['OS_PASSWORD'] = passwd



    username = india_os['OS_USERNAME']
    password = india_os['OS_PASSWORD']
    authurl = india_os['OS_AUTH_URL']
    tenant = india_os['OS_TENANT_NAME']

    print password
    '''
    username = os.environ['OS_USERNAME']
    password = os.environ['OS_PASSWORD']
    authurl = os.environ['OS_AUTH_URL']
    '''
    india = cloud_openstack("india", authurl, tenant, username, password)

    india._vm_show("gvonlasz-001")
    india.dump()
    india._vm_show("gvonlasz-001")
    india.dump()
    """



#! /usr/bin/env pytho

#
# see also http://docs.openstack.org/cli/quick-start/content/nova-cli-reference.html
#

import iso8601
import sys
sys.path.insert(0, '../..')

from datetime import datetime
import pprint
pp = pprint.PrettyPrinter(indent=4)
import json
import os

from sh import fgrep
import novaclient
from sh import nova

# from cm_credential import credentials
from cloudmesh.openstack.cm_table import table as cm_table
from cloudmesh.cm_config import cm_config

from novaclient.v1_1 import client


class openstack:

    type = "openstack"   # global var
    flavors = {}         # global var
    images = {}          # global var
    servers = {}         # global var
    credential = None    # global var
    label = None         # global var

    cloud = None         # internal var
    user_id = None       # internal var

    _nova = nova

    def vms(self):
        return self.servers

    def credentials(self, cred):
        self.credential = cred

    ######################################################################
    # initialize
    ######################################################################
    def __init__(self,
                 label,
                 authurl=None,
                 project=None,
                 username=None,
                 password=None):
        """
        initializes the openstack cloud from a defould novaRC file
        locates at ~/.futuregrid.org/openstack. However if the
        parameters are provided it will instead use them
        """

        self.clear()
        self.label = label
        self.config(label,
                    authurl=authurl,
                    project=project,
                    username=username,
                    password=password)
        self.connect()

    def clear(self):
        """
        clears the data of this openstack instance, a new connection
        including reading the credentials and a refresh needs to be
        called to obtain again data.
        """
        self.type = "openstack"
        self.flavors = {}
        self.images = {}
        self.servers = {}
        self.credential = None
        self.cloud = None
        self.user_id = None

    def connect(self):
        """
        establishes a connection to the OpenStack cloud,
        e.g. initializes the needed components to conduct subsequent
        queries.
        """
        self.cloud = client.Client(
            self.credential['OS_USERNAME'],
            self.credential['OS_PASSWORD'],
            self.credential['OS_TENANT_NAME'],
            self.credential['OS_AUTH_URL']
        )

    def config(self, label,
               authurl=None,
               project=None,
               username=None,
               password=None):
        """
        reads in the configuration file if specified, and does some
        internal configuration.
        """
        self.label = label
        if username == None:
            config = cm_config()
            self.credential = config.get(label)
            print self.credential
            self.user_id = self.credential['OS_USER_ID']
            # self.credential = credentials(label)
        else:
            self.credential = {}
            self.credential['OS_USERNAME'] = username
            self.credential['OS_PASSWORD'] = password
            self.credential['OS_AUTH_URL'] = authurl
            self.credential['label'] = label
            self.credential['OS_TENANT_NAME'] = project
        """
        self._nova = nova.bake("--os-username",    self.credential.username,
                               "--os-password",    self.credential.password,
                               "--os-auth-url",    self.credential.url,
                               "--os-tenant-name", self.credential.project)
        """

    ######################################################################
    # find userid
    ######################################################################

    def intro(self, what):
        """ usde to find some methods form novaclient"""
        import inspect

        print 70 * "="
        print "class ", what.__class__.__name__, ":"
        list = inspect.getmembers(what, predicate=inspect.ismethod)
        for element in list:
            print "    ", element[0]
        try:
            pp.pprint(what.__dict__)
        except:
            return

    def novaclient_dump(self):

        # playing around to discover methods
        self.intro(self.cloud)
        self.intro(self.cloud.services)
        self.intro(self.cloud.servers)
        self.intro(self.cloud.client)

        self.intro(self.cloud.agents)
        self.intro(self.cloud.aggregates)
        self.intro(self.cloud.availability_zones)
        self.intro(self.cloud.certs)
        self.intro(self.cloud.client)
        self.intro(self.cloud.cloudpipe)
        self.intro(self.cloud.coverage)
        self.intro(self.cloud.dns_domains)
        self.intro(self.cloud.dns_entries)
        self.intro(self.cloud.fixed_ips)
        self.intro(self.cloud.flavor_access)
        self.intro(self.cloud.flavors)
        self.intro(self.cloud.floating_ip_pools)
        self.intro(self.cloud.floating_ips)
        self.intro(self.cloud.floating_ips_bulk)
        self.intro(self.cloud.fping)
        self.intro(self.cloud.hosts)
        self.intro(self.cloud.hypervisors)
        self.intro(self.cloud.images)
        self.intro(self.cloud.keypairs)
        self.intro(self.cloud.limits)
        self.intro(self.cloud.networks)
        self.intro(self.cloud.os_cache)
        self.intro(self.cloud.project_id)
        self.intro(self.cloud.quota_classes)
        self.intro(self.cloud.quotas)
        self.intro(self.cloud.security_group_rules)
        self.intro(self.cloud.security_groups)
        self.intro(self.cloud.servers)
        self.intro(self.cloud.services)
        self.intro(self.cloud.usage)
        self.intro(self.cloud.virtual_interfaces)
        self.intro(self.cloud.volume_snapshots)
        self.intro(self.cloud.volume_types)
        self.intro(self.cloud.volumes)

        now = datetime.now()

    def find_user_id(self):
        """
        this method returns the user id and stores it for later use.
        """
        # As i do not know how to do this properly, we just create a
        # VM and than get the userid from there

        self.cloud.ensure_service_catalog_present(self.cloud)
        catalog = self.cloud.client.service_catalog.catalog
        pp.pprint(catalog['access']['user'], "User Credentials")
        pp.pprint(catalog['access']['token'], "Token")

        print(result)

        sys.exit()
        if self.user_id == None:
            sample_flavor = self.cloud.flavors.find(name="m1.tiny")
            sample_image = self.cloud.images.find(
                id="6d2bca76-8fff-4d57-9f29-50378539b4fa")
            sample_vm = self.cloud.servers.create(
                "%s-id" % self.credential["OS_USERNAME"],
                flavor=sample_flavor,
                image=sample_image)
            self.user_id = sample_vm.user_id
            sample_vm.delete()
        return self.user_id

    ######################################################################
    # print
    ######################################################################

    def __str__(self):
        """
        print everything but the credentials that is known about this
        cloud in json format.
        """
        information = {
            'label': self.label,
            'flavors': self.flavors,
            'vms': self.servers,
            'images': self.images}
        return json.dumps(information, indent=4)

    ######################################################################
    # get methods
    ######################################################################

    def type():
        return self.type

    ######################################################################
    # refresh
    ######################################################################

    """
        def _get_image_dict():
                return _self.cloud.images.list(detailed=True)

        def _update_image_dict(image):
                del image.manager
                del image._info
                del image._loaded
                # del information.links
                return (image.id, image)

    """
    
    
    def refresh(self, type=None):

        time_stamp = datetime.now().strftime('%Y-%m-%dT%H-%M-%SZ')
        selection = ""
        if type:
            selection = type.lower()[0]
            all = selection == 'a'
        else:
            all = True

        if selection == 'i' or all:
            # list = _get_image_dict()
            list = self.cloud.images.list(detailed=True)
            for information in list:
                # (id, image) = _update_image_dict(information)
                image = information.__dict__
                del image.manager
                del image._info
                del image._loaded
                # del information.links
                self.images[information.id] = image
                self.images[information.id]['cm_refresh'] = time_stamp

        if selection == 'f' or all:
            # list = _get_floavors_list()
            list = self.cloud.flavors.list()
            for information in list:
                #flavor = _update_flavors_dict(information)
                flavor = information.__dict__
                # clean not neaded info
                del flavor.manager
                del flavor._info
                del flavor._loaded
                # del information.links
                self.flavors[information.name] = flavor
                self.flavors[information.name]['cm_refresh'] = time_stamp

        if selection == 'v' or selection == None or all:
            #list = _get_servers_list()
            list = self.cloud.servers.list(detailed=True)

            for information in list:
                # vm = _update_servers_dict(information)
                vm = information.__dict__
                #pp.pprint(vm)
                delay = vm['id']
                del vm['manager']
                del vm['_info']
                del vm['_loaded']
                # del information.links

                self.servers[information.id] = vm
                self.servers[information.id]['cm_refresh'] = time_stamp

    ######################################################################
    # create a vm
    ######################################################################
    def vm_create(self, name, flavor_name, image_id):
        """
        create a vm
        """

        vm_flavor = self.cloud.flavors.find(name=flavor_name)
        vm_image = self.cloud.images.find(id=image_id)
        vm = self.cloud.servers.create(name,
                                       flavor=vm_flavor,
                                       image=vm_image
                                       )
        delay = vm.user_id  # trick to hopefully get all fields
        data = vm.__dict__
        del data['manager']
        # data['cm_name'] = name
        # data['cm_flavor'] = flavor_name
        # data['cm_image'] = image_id
        return {str(data['id']):  data}

    ######################################################################
    # delete vm(s)
    ######################################################################

    def vm_delete(self, id):
        """
        delete a single vm and returns the id
        """

        vm = self.cloud.servers.delete(id)
        # return just the id or None if its deleted
        return vm

    def vms_delete(self, ids):
        """
        delete many vms by id. ids is an array
        """
        for id in ids:
            print "Deleting %s" % self.servers[id]['name']
            vm = self.vm_delete(id)

        return ids

    ######################################################################
    # list user images
    ######################################################################

    def vms_user(self, refresh=False):
        """
        find my vms
        """
        user_id = self.find_user_id()

        time_stamp = datetime.now().strftime('%Y-%m-%dT%H-%M-%SZ')
        if refresh:
            self.refresh("images")

        result = {}
        for (key, vm) in self.servers.items():
            if vm['user_id'] == self.user_id:
                result[key] = vm

        return result

    ######################################################################
    # list project vms
    ######################################################################

    def vms_project(self, refresh=False):
        """
        find my vms
        """
        user_id = self.find_user_id()

        time_stamp = datetime.now().strftime('%Y-%m-%dT%H-%M-%SZ')
        if refresh:
            self.refresh("images")

        result = {}
        for (key, vm) in self.servers.items():
            result[key] = vm

        return result

    ######################################################################
    # delete images from a user
    ######################################################################

    def vms_delete_user(self):
        """
        find my vms and delete them
        """

        ids = cloud.find('user_id')
        cloud.vms_delete(ids)

        return

    ######################################################################
    # find
    ######################################################################

    def find(self, key, value=None):
        ids = []
        if key == 'user_id' and value == None:
            value = self.user_id
        for (id, vm) in self.servers.items():
            if vm[str(key)] == value:
                ids.append(str(vm['id']))

        return ids

    ######################################################################
    # rename
    ######################################################################

    def rename(self, old, new, id=None):
        all = self.find('name', old)
        print all
        if len(all) > 0:
            id = all[0]
            vm = self.cloud.servers.update(id, new)
        return

    def reindex(self, prefixold, prefix, index_format):
        all = self.find('user_id')
        counter = 1
        for id in all:
            old = self.servers[id]['name']
            new = prefix + index_format % counter
            print "Rename %s -> %s, %s" % (old, new, self.servers[id]['key_name'])
            if old != new:
                vm = self.cloud.servers.update(id, new)
            counter += 1

    def reindex(self, prefix, index_format):
        all = self.find('user_id')
        counter = 1
        for id in all:
            old = self.servers[id]['name']
            new = prefix + index_format % counter
            print "Rename %s -> %s, %s" % (old, new, self.servers[id]['key_name'])
            # if old != new:
            #    vm = self.cloud.servers.update(id, new)
            counter += 1

    ######################################################################
    # TODO
    ######################################################################
    """
    refresh just a specific VM
    delete all images that follow a regualr expression in name
    look into sort of images, flavors, vms
    """

    ######################################################################
    # EXTRA
    ######################################################################

    def table_col_to_dict(self, body):
        result = {}
        for element in body:
            key = element[0]
            value = element[1]
            result[key] = value
        return result

    def Table_matrix(self, text, format=None):
        lines = text.splitlines()
        headline = lines[0].split("|")
        headline = headline[1:-1]
        for i in range(0, len(headline)):
            headline[i] = str(headline[i]).strip()

        lines = lines[1:]

        body = []

        for l in lines:
            line = l.split("|")
            line = line[1:-1]
            entry = {}
            for i in range(0, len(line)):
                line[i] = str(line[i]).strip()
                if format == "dict":
                    key = headline[i]
                    entry[key] = line[i]
            if format == "dict":
                body.append(entry)
            else:
                body.append(line)
        if format == 'dict':
            return body
        else:
            return (headline, body)

    ######################################################################
    # CLI call of ussage
    ######################################################################
    def parse_isotime(self, timestr):
        """Parse time from ISO 8601 format"""
        try:
            return iso8601.parse_date(timestr)
        except iso8601.ParseError as e:
            raise ValueError(e.message)
        except TypeError as e:
            raise ValueError(e.message)

    def usage(self, start, end, format='dict'):
        """ returns the usage information of the tennant"""

        # print 70 * "-"
        # print self.cloud.certs.__dict__.get()
        # print 70 * "-"

        tenantid = "member"  # not sure how to get that
        iso_start = self.parse_isotime(start)
        iso_end = self.parse_isotime(end)
        print ">>>>>", iso_start, iso_end
        info = self.cloud.usage.get(tenantid, iso_start, iso_end)

        # print info.__dict__
        sys.exit()

        result = fgrep(nova("usage", "--start", start, "--end", end), "|")
        (headline, matrix) = self.table_matrix(result)
        headline.append("Start")
        headline.append("End")
        matrix[0].append(start)
        matrix[0].append(end)

        if format == 'dict':
            result = {}
            for i in range(0, len(headline)):
                result[headline[i]] = matrix[0][i]
            return result
        else:
            return (headline, matrix[0])

    ######################################################################
    # CLI call of absolute-limits
    ######################################################################
    def limits(self):
        """ returns the usage information of the tennant"""

        list = []

        info = self.cloud.limits.get()
        del info.manager
        rates = info.__dict__['_info']['rate']

        for rate in rates:
            limit_set = rate['limit']
            print limit_set
            for limit in limit_set:
                list.append(limit)

        return list


##########################################################################
# MAIN FOR TESTING
##########################################################################

if __name__ == "__main__":

    credential_test = False
    flavor_test = False
    table_test = False
    image_test = False
    vm_test = True
    cloud_test = False
    usage_test = False

    if credential_test:
        credential = cm_config('india-openstack')
        # print credential

    cloud = openstack("india-openstack")


    cloud.novaclient_dump()

    if usage_test:
        #print json.dumps(cloud.usage("2000-01-01T00:00:00", "2013-12-31T00:00:00"), indent=4)

        print json.dumps(cloud.limits(), indent=4)

                 
        # print json.dumps(cloud.usage("2000-01-01", "2013-12-31"), indent=4)
        # print json.dumps(cloud.usage("2000-01-01", "2013-12-31",format=None), indent=4)

        # print json.dumps(cloud.limits(format='dict'), indent=4)
        # print json.dumps(cloud.limits(format='array'), indent=4)

    if flavor_test or table_test:
        cloud.refresh('flavors')
        print json.dumps(cloud.flavors, indent=4)

    if table_test:
        table = cm_table()
        columns = ["id", "name", "ram", "vcpus"]

        table.create(cloud.flavors, columns, header=True)
        print table

        table.create(cloud.flavors, columns, format='HTML', header=True)
        print table

        table.create(cloud.flavors, columns, format='%12s', header=True)
        print table

    if image_test:
        cloud.refresh('images')
        print json.dumps(cloud.images, indent=4)

    if vm_test:
        cloud.refresh('vms')
        print json.dumps(cloud.images, indent=4)

    if cloud_test:
        cloud.refresh()
        print cloud

        #print cloud.find_user_id()


    
    


    name ="%s-%04d" % (cloud.credential["OS_USERNAME"], 1)

    out = cloud.vm_create(name, "m1.tiny", "6d2bca76-8fff-4d57-9f29-50378539b4fa")
    pp.pprint(out)
    print json.dumps(out, indent=4)

    """
    key = out.keys()[0]
    id = out[key]["id"]

    print ">>>", id

    vm = cloud.vm_delete(id)
    print vm


    list = cloud.vms_user()
    print json.dumps(list, indent=4)

    print cloud.find_user_id()

    list = cloud.vms_delete_user()
    print list
    """
    """
    for i in range (1,3):
        name ="%s-%04d" % (cloud.credential["OS_USERNAME"], i)
        out = cloud.vm_create(name, "m1.tiny", "6d2bca76-8fff-4d57-9f29-50378539b4fa")
        pp.pprint(out)
    """

    """
    print cloud.find('name', name)
    """

    #ids = cloud.find('user_id')

    #print ids

    #cloud.vms_delete(ids)

    #print cloud.vms_delete_user()

    # cloud.rename("gvonlasz-0001","gregor")

    #cloud.reindex("deleteme-", "%03d")



import sys
sys.path.insert(0, '..') 
from datetime import datetime
import pprint 
pp = pprint.PrettyPrinter(indent=4)
import json
import os

from libcloud.compute.types import Provider
from libcloud.compute.providers import get_driver
from libcloud.compute.types import NodeState

import libcloud.security
import time
from sh import fgrep

#from openstack.cm_table import table as cm_table
from cloudmesh.cm_config import cm_config

class eucalyptus:
    """
    requires a cloudmesh yaml file with the following structure:

    default: sierra-openstack

    cloudmesh:

        india-eucalyptus:
            BASEDIR: ~/.futuregrid/india/eucalyptus
            host: 127.127.127.127
            port: 8773
            fg-82:
                EC2_PRIVATE_KEY: euca2-user-05.....-pk.pem
                EC2_CERT: euca2-user-0429....-cert.pem
                EUCALYPTUS_CERT: cloud-cert.pem
                EC2_ACCOUNT_NUMBER: '709......'
                EC2_ACCESS_KEY: 'GF78F0E7......'
                EC2_SECRET_KEY: 'yf07e87fe.......'
                EC2_USER_ID: '09876....'

    """

    type = "eucalyptus"
    sizes = {}
    images = {}
    nodes = {}
    cloud = None
    label = None
    credentials = {}

    filename = "%(home)s/%(location)s" % {
        "home" : os.environ['HOME'], 
        "location" : ".futuregrid/cloudmesh.yaml"
        }
   
    def vms(self):
        return self.servers

   
    def __init__(self, label,
                 project=None,
                 accessKey=None,
                 secretKey=None):
        """
        initializes the openstack cloud from a defould novaRC file
        locates at ~/.futuregrid.org/openstack. However if the
        parameters are provided it will instead use them
        """
        self.clear()
        self.config(label,project,accessKey,secretKey)
        self.connect()

    def clear(self):
        """
        clears the data of this openstack instance, a new connection
        including reading the credentials and a refresh needs to be
        called to obtain again data.
        """
        type = "eucalyptus"
        self.sizes = {}
        self.images = {}
        self.nodes = {}
        self.credentials = {}
        self.cloud = None
        self.user_id = None
    
    def connect(self):
                
        Driver = get_driver(Provider.EUCALYPTUS)
        self.cloud = Driver(self.credentials['accessKey'], 
                            self.credentials['secretKey'],
                            host="149.165.146.135", 
                            secure=False, port=8773,
                            path="/services/Eucalyptus")
      

    def config(self,label=None,project=None,accessKey=None,secretKey=None):
        """
        reads in the configuration file if specified, and does some
        internal configuration.
        """ 
        if label == None and accessKey == None:
            label = 'india-eucalyptus'
            project = 'fg-82'

        if accessKey == None:
            self.label = label

            config = cm_config()
            configuration = config.get(label)
            
            pp.pprint(configuration)

            basedir = configuration['BASEDIR'] = configuration['BASEDIR'].replace('~',os.environ['HOME'])
            configuration[project]['EC2_PRIVATE_KEY']  = "%s/%s" % (basedir, configuration[project]['EC2_PRIVATE_KEY'] )
            configuration[project]['EUCALYPTUS_CERT']  = "%s/%s" % (basedir, configuration[project]['EUCALYPTUS_CERT'] )

            pp.pprint(configuration)


            self.credentials['accessKey']=configuration[project]['EC2_ACCESS_KEY']
            self.credentials['secretKey']=configuration[project]['EC2_SECRET_KEY']

        else:
            self.credentials['accessKey']=accessKey
            self.credentials['secretkey']=secretKey
   
        
    def __str__ (self):
        """
        print everything but the credentials that is known about this
        cloud in json format.
        """
        information = {
            'label': self.label,
            'sizes': self.sizes,
            'servers': self.nodes,
            'images' : self.images}
        return json.dumps(information, indent=4)

    def type():
        return self.type

    def refresh(self, type=None):

        time_stamp = datetime.now().strftime('%Y-%m-%dT%H-%M-%SZ')
        selection = ""
        if type:
            selection = type.lower()[0]
            all = selection == 'a'
        else:
            all = True
        
        if selection == 'i' or all:

            list = self.cloud.list_images()


            for information in list:
                image=information.__dict__
                del information._uuid
                del information.driver
                self.images[information.id] = image
                self.images[information.id]['cm_refresh'] = time_stamp
                
        if selection == 'n' or all:

            list = self.cloud.list_nodes()


            for information in list:
                node=information.__dict__
                del information._uuid
                del information.driver
                self.nodes[information.id] = node
                self.nodes[information.id]['cm_refresh'] = time_stamp
        if selection == 's' or all:

            list = self.cloud.list_sizes()


            for information in list:
                size=information.__dict__
                #del information.virtualizationtype
                del information.driver
                del information._uuid
                self.sizes[information.id] = size
                self.sizes[information.id]['cm_refresh'] = time_stamp


    def getNodebyID(self,nodeid):        
        nodes = self.cloud.list_nodes()      
        for node in nodes :
            if node.id == nodeid :
                return node   
 
    def vm_delete(self,node):
        result = destroy_node(self, node)
            # add result to internal cache
        print ("vm Deleted")
        return result
    
    def restart(self,node):
        """restarts a vm with the given name"""
        result = reboot_node(self, node)
        # add result to internal cache
        print result
      

    def vm_create(self, name, flavor_name, image_id):
        """
        create a vm
        """
        flavours = self.cloud.list_sizes()
        vm_flavour= flavours[0]
        for flavour in flavours :
            if flavour.id == 'm1.small' :
                vm_flavour= flavour

        images = self.cloud.list_images()
        vm_image = images[0]
        for image in images :
            if image.id == 'ami-000000b4' :
                vm_image = image

        if (name== None):
            vm_name=self._generate_vmname(randint(0,1000))
        else:
            vm_name=name;
     
        self.vm = self.cloud.create_node(name=vm_name, image=vm_image, size=vm_flavour);
        
        while 1 :
            time.sleep(15)
            updatedNode = self.getNodebyID(self.vm.id)
            if updatedNode.state == 3 :
                print "pending " + updatedNode.id
            if updatedNode.state == 0 :
                print("successful creation of vm")
                time.sleep(60)
                break;
            if updatedNode.state == 4 :
                print(node.id, "Has Errored out");
                nodes.remove(vm)
                break;

        data = self.vm.__dict__
        del data['driver']
        pp.pprint(data)
        return {}


    def _generate_vmname(self,index):
        number = str(index).zfill(3)
        name = '%s-%s' % (self.credentials['username'], number)
        return name




                
if __name__ == "__main__":


  credential_test = False
  sizes_test = False
  table_test = False
  image_test = False
  nodes_test = False
  cloud_test = True

  cloud = eucalyptus('india-eucalyptus', 'fg-82')

  if image_test:
      cloud.refresh('images')
      print json.dumps(cloud.images, indent=4)
#-      pp.pprint (cloud.images)
  
  if nodes_test:
      cloud.refresh('nodes')
      print json.dumps(cloud.nodes, indent=4)
     
  if sizes_test:
      cloud.refresh('sizes')
      print json.dumps(cloud.sizes, indent=4)

  if cloud_test:
      cloud.refresh()
      print cloud



""" run with

nosetests -v --nocapture

or

nosetests -v

"""
import sys
#sys.path.insert(0, '..')

from cm_keys import cm_keys
import json
import pprint
pp = pprint.PrettyPrinter(indent=4)

header = True

def HEADING(txt):
    if header:
        print
        print "#", 70 * '#'
        print "#", txt
        print "#", 70 * '#'

class Test_cloudmesh:

    #filename = None
    #filename = "credentials-example-keys.yaml"
    filename = "$HOME/.futuregrid/cloudmesh-new.yaml"


    def setup(self):
        self.keys = cm_keys(self.filename)

    def tearDown(self):
        pass

    def test00_file(self):
        HEADING("00 FILE")
        try:
            self.keys = cm_keys("wrong file")
        except:
            pass
            

    def test01_print(self):
        HEADING("01 PRINT")
        print self.keys
        pass
    
    def test02_names(self):
        HEADING("02 NAMES")
        print self.keys.names()
        
    def test03_default(self):
        HEADING("03 DEFAULT")
        print self.keys.default()

    def test04_getvalue(self):
        HEADING("04 GET VALUE")
        print self.keys._getvalue("gregor")
        print self.keys._getvalue("default")
        print self.keys._getvalue("keys")

    def test05_set(self):
        HEADING("05 SET DEFAULT")
        self.keys.setdefault("gregor")
        print self.keys.default()

    def test06_get(self):
        HEADING("06 GET GREGOR")
        print self.keys["gregor"]

    def test07_get(self):
        HEADING("07 GET")
        print self.keys["default"]
        print self.keys["gregor"]

    def test06_set(self):
        HEADING("06 SET HELLO WORLD")

        print self.keys["keys"]

        self.keys["gregor"] = "~/.ssh/id_rsa.pub"
        self.keys["hello"] = "~/.ssh/id_rsa"
        
        print self.keys["keys"]

        self.keys["default"] = "hello"
        print self.keys["keys"]
                
        #assert (self.keys._getvalue("default") == "hello") and (self.keys._getvalue("gregor") == "world")

    def test07_type(self):
        HEADING("TYPE")
        print "LLL", self.keys.type("gregor") 
        for name in self.keys.names():
            print name
            value =  self.keys[name]
            print self.keys.type(name), name, value
            
        assert (self.keys.type("gregor") == "file")

    def test08_fingerprint(self):
        HEADING("FINGERPRINT")
        print self.keys.fingerprint("gregor")
        print self.keys.fingerprint("default") 






import sys
sys.path.insert(0, '..')
import yaml
#import pyaml
import os
import stat
import json
import collections

from string import Template


def path_expand(text):
    """ returns a string with expanded variavble """
    template = Template(text)
    result = template.substitute(os.environ)
    return result


class cm_config(object):

    ######################################################################
    # global variables
    ######################################################################

    default_path = '.futuregrid/cloudmesh.yaml'
    yaml_template = 'cloudmesh-template.yaml'
    filename = ""
    data = collections.OrderedDict()

    ######################################################################
    # initialization methods
    ######################################################################

    def __init__(self, filename=None):
        if filename == None:
            home = os.environ['HOME']
            self.filename = "%s/%s" % (home, self.default_path)
        else:
            self.filename = filename
        self.read(self.filename)
        self._userdata_handler = None
        self._cloudcreds_handler = None

    @property
    def userdata_handler(self):
        """Plug-in class that knows how to get all the user/project data"""
        return self._userdata_handler

    @userdata_handler.setter
    def userdata_handler(self, value):
        self._userdata_handler = value

    @property
    def cloudcreds_handler(self):
        """Plug-in class that knows how to get all the cloud credential data"""
        return self._cloudcreds_handler

    @cloudcreds_handler.setter
    def cloudcreds_handler(self, value):
        self._cloudcreds_handler = value


    ######################################################################
    # Methods to initialize (create) the config data
    ######################################################################
    def _initialize_user(self, username):
        """Loads user data, including profile, projects, and credentials"""
        user = self.userdata_handler(username)

        self.data['cloudmesh']['prefix'] = username
        self.data['cloudmesh']['index'] = "001"
        

        self.data['cloudmesh']['profile'] = {
            'username': username,
            'uid': user.uid,
            'gid': user.gid,
            'firstname': user.firstname,
            'lastname': user.lastname,
            'phone': user.phone,
            'e-mail': user.email,
            'address': user.address
            }

        keys = { 'default': None, 'keylist': {} }
        for key in user.keys.keys():
            if keys['default'] is None:
                keys['default'] = key
            keys['keylist'][key] = user.keys[key]
        self.data['cloudmesh']['keys'] = keys

        self.data['cloudmesh']['projects'] = {
            'active': user.activeprojects,
            'completed': user.completedprojects,
            'default': user.defaultproject
            }
                
        self.data['cloudmesh']['active'] = user.activeclouds
        self.data['cloudmesh']['default'] = user.defaultcloud


    def _initialize_clouds(self):
        """Creates cloud credentials for the user"""
        self.data['cloudmesh']['clouds'] = {}
        cloudlist = self.active()
        for cloud in cloudlist:
            cloudcreds = self.cloudcreds_handler(self.profile(), self.projects('default'), self.projects('active'), cloud)
            cloudcreds.initialize_cloud_user()
            self.data['cloudmesh']['clouds'][cloud] = cloudcreds.data


    def initialize(self, username):
        """Creates or resets the data for a user.  Note that the
        userdata_handler and cloudcreds_handler properties must be set
        with appropriate handler classes."""
        self.data = yaml.safe_load(open(self.yaml_template, "r"))
        self._initialize_user(username)
        self._initialize_clouds()

    ######################################################################
    # read and write methods
    ######################################################################

    def read(self, filename):
        self.filename = filename
        if os.path.exists(filename):
            f = open(self.filename, "r")
            self.data = yaml.safe_load(f)
            f.close()

    def write(self, filename=None):

        #pyaml.dump(self.data, f, vspacing=[2, 1, 1])
        text = yaml.dump(self.data, default_flow_style=False)
        # avoiding to think about regexp ;-)
        # shold be eplace all "\n  ." with "\\n  ." but not when . is -   
        text = text.replace("\n  ", "\n\n  ")
        text = text.replace("\n\n  -", "\n  -")
        text = text.replace("\n\n    ", "\n    ")
        text = text.replace("\n\n      ", "\n      ")
        text = text.replace("\n\n        ", "\n        ")

        fpath = filename or self.filename
        f = os.open(fpath, os.O_CREAT | os.O_EXCL | os.O_WRONLY, stat.S_IRUSR | stat.S_IWUSR)
        os.write(f, text)
        if os.geteuid() == 0:
            os.fchown(f, int(self.profile()['uid'] or -1),
                      int(self.profile()['gid'] or -1))
        os.close(f)

        
    ######################################################################
    # print methods
    ######################################################################

    def __str__(self):
        return json.dumps(self.data, indent=4)

    def export_line(self, attribute, value):
        return "export %s=%s\n" % (attribute, value)

    ######################################################################
    # get methods
    ######################################################################

    def incr(self, value=1):
        self.data['cloudmesh']['index'] = int(self.data['cloudmesh']['index']) + int(value)
    @property
    def vmname(self):
        return "%s-%04d" % (self.data['cloudmesh']['prefix'], int(self.data['cloudmesh']['index']))

    def default(self):
        return self.data['cloudmesh']['default']

    def active(self):
        return self.data['cloudmesh']['active']

    @property
    def prefix(self):
        return self.data['cloudmesh']['prefix']

    @prefix.setter
    def prefix(self, value):
        self.data['cloudmesh']['prefix'] = value
        
    @property
    def index(self):
        return self.data['cloudmesh']['index']

    @index.setter
    def index(self, value):
        self.data['cloudmesh']['index'] = int(value)

    def profile(self):
        return self.data['cloudmesh']['profile']

    def userkeys(self, attribute=None, expand=True):
        if attribute == None:
            return self.data['cloudmesh']['keys']
        else:
            if attribute == 'default':
                name = self.data['cloudmesh']['keys']['default']
                value = self.data['cloudmesh']['keys']['keylist'][name]
            else:
                value = self.data['cloudmesh']['keys']['keylist'][attribute]
            if expand:
                value = path_expand(value)
            return value

    def default(self,cloudname):
        return self.data['cloudmesh']['clouds'][cloudname]['default']

    def projects(self, status):
        return self.data['cloudmesh']['projects'][status]

    def clouds(self):
        return self.data['cloudmesh']['clouds']

    def cloud(self, cloudname):
        return self.data['cloudmesh']['clouds'][cloudname]
        
    def cloud_default(self, cloudname, defname):
        cloud = self.cloud(cloudname)
        defaults = cloud['default'] if 'default' in cloud else []
        return defaults[defname] if defname in defaults else None


    def get(self, key=None, expand=False):
        if key == None:
            return self.data['cloudmesh']
        else:
            if expand:
                d = self.cloud(key)['credentials']
                for key in d:
                    d[key] = path_expand(d[key])
                return d
            else:
                return self.cloud(key)['credentials']

    # This method may not be exactly what I think it is, but based on usage it
    # appears as if it is supposed to get the keys of the clouds

    def keys(self):
        return self.clouds().keys()

    def rc(self, name):
        result = self.get(name)
        lines = ""
        for (attribute, value) in result.iteritems():
            lines += self.export_line(attribute, value)
        return lines

    def rc_euca(self, name, project):
        result = self.cloud(name)
        eucakeydir = 'EUCA_KEY_DIR'
        lines = self.export_line(eucakeydir, result[eucakeydir])

        for (attribute, value) in result.iteritems():
            if attribute != eucakeydir:
                if type(value) is dict:
                    if attribute == project:
                        for (pattribute, pvalue) in value.iteritems():
                            lines += self.export_line(pattribute, pvalue)
                else:    
                    lines += self.export_line(attribute, value)
        return lines

##########################################################################
# MAIN METHOD FOR TESTING
##########################################################################

if __name__ == "__main__":
    config = cm_config()

    print config

    print "================="
    print config.projects('active')
    print config.projects('completed')
    print "================="
    print config.get('india-openstack')
    print "================="
    print config.get()
    print "================="
    print config.keys()
    print "================="
    print config.rc('india-openstack')
    print "================="
    print config.default()
    print "================="
    outfile = "%s/%s" % (os.environ['HOME'], ".futuregrid/junk.yaml")
    print config.write(outfile)
    os.system("cat " + outfile)
    print "================="
    configuration = config.get('azure')

    print configuration['username']

    print "================="

    configuration = config.get('india-eucalyptus')
    print configuration

    print configuration['host']



#################
# SEEMS NO ONE USING THIS FILE. CAN WE MOVE THIS TO DEPRECATED FOLDER?
# EXPIRATION DATE: 30 days after, this file will be moved to deprecated unless
# there is any claim.
#
# Hyungro Lee (hroe.lee@gmail)
#################
import os
# cloud mesh
from flask import Flask, render_template, request
from flask_flatpages import FlatPages
#import cloud_mesh as fg
from cloudmesh.cloudmesh_new import  cloudmesh
from datetime import datetime
#from sh import cm 


DEBUG = True
FLATPAGES_AUTO_RELOAD = DEBUG
FLATPAGES_EXTENSION = '.md'

filename = "VERSION.txt"
version = open(filename).read()

app = Flask(__name__)
app.config.from_object(__name__)
pages = FlatPages(app)

# @hroelee (hyungro) commented March 1st, 2013
# Personally, I prefer to use class variables instead of using global variables.
# Example of using class in Flask: https://github.com/futuregrid/cloud-metrics/blob/master/fgws/FGWSApps.py
#
# Opinion from stackoverflow: Using global variables in a function other than the one that created them
# http://stackoverflow.com/questions/423379/using-global-variables-in-a-function-other-than-the-one-that-created-them
#
#clouds = fg.cloud_mesh()
clouds = cloudmesh()
clouds.load()
# AttributeError: cloudmesh instance has no attribute 'refresh'
#clouds.refresh()
# TEST CASE
clouds.refresh("openstack")

clouds.clouds

def make_active(name):
  active = {'home' : "", 
            'table' : "", 
            'contact' : "", 
            "metric" : ""}
  active[name] = 'active'
  return active


@app.route('/')
def index():
    active=make_active('home')
    return render_template('index.html', 
                           pages=pages, 
                           active=active,
                           version=version)

@app.route('/cm/refresh/')
@app.route('/cm/refresh/<cloud>/')
def refresh(cloud=None, server=None):
  print "-> refresh", cloud, server
  global clouds
  #AttributeError: cloudmesh instance has no attribute 'refresh'
  #clouds.refresh()
  clouds.save()
  return table()

@app.route('/cm/kill/')
def kill_vms():
  print "-> kill all"
  r = cm("--set", "quiet", "kill", _tty_in=True)
  return table()


@app.route('/cm/delete/<cloud>/<server>/')
def delete_vm(cloud=None, server=None):
  print "-> delete", cloud, server
  if (cloud == 'india'):
    r = cm("--set", "quiet", "delete:1", _tty_in=True)
  return table()

@app.route('/cm/start/<cloud>/')
def start_vm(cloud=None, server=None):
  print "-> start", cloud, server
  if (cloud == 'india'):
    r = cm("--set", "quiet", "start:1", _tty_in=True)
  return table()

'''
#gregorss test 
@app.route('/cm/metric/<startdate>/<enddate>/<host>')
def list_metric(cloud=None, server=None):
    print "-> generate metric", startdate, endadte
    #r = fg-metric(startdate, enddate, host, _tty_in=True)
    return render_template('metric1.html', 
                           startdate=startdate,
                           active=active,
                           version=version,
                           endate=enddate)
    #return table()
'''

@app.route('/save/')
def save():
  print "Saving the cloud status"
  global clouds
  clouds.save()
  return table()

@app.route('/load/')
def load():
  print "Loading the cloud status"
  global clouds
  clouds.load()
  return table()

@app.route('/table/')
def table():
    global clouds

    active=make_active('table')
    time_now = datetime.now().strftime("%Y-%m-%d %H:%M")

    # note thet call to sierra is fake it just goes to india and sets cloudname to sierra.
    #clouds.dump()
    #keys = clouds.get_keys()
    return render_template('table.html', 
                           updated = time_now,
                           keys="",##",".join(clouds.get_keys()),
                           clouds=clouds.clouds,
                           image='myimage',
                           pages=pages,
                           active=active,
                           version=version)

#@app.route('/metric/<s_date>/<e_date>/<user>/<cloud>/<host>/<period>/<metric>')
@app.route('/metric/main', methods=['POST','GET'])
def metric():
    global clouds
    args = {"s_date":request.args.get('s_date',''),
            "e_date":request.args.get('e_date',''),
            "user":request.args.get('user',''),
            "cloud":request.args.get('cloud',''),
            "host":request.args.get('host',''),
            "period":request.args.get('period',''),
            "metric":request.args.get('metric','')}

    return render_template('metric.html', 
                           clouds=clouds.get(),
                           metrics=clouds.get_metrics(args),
                           pages=pages,
                           active=make_active('metric'),
                           version=version)

@app.route('/<path:path>/')
def page(path):
    active=make_active(str(path))
    page = pages.get_or_404(path)
    return render_template('page.html', 
                           page=page, 
                           pages=pages, 
                           active=active, 
                           version=version)

if __name__ == '__main__':
    app.debug = True
    app.run()



class mock_user:
    """ Provides a mock class for testing. Represents the user/project
    data that will come from LDAP and/or Portal. """

    def __init__(self, username):
        self._username = username
        self._data = None

    def _load_data(self):
        self._data = {
            'profile': {
                'username': self._username,
                'uid': '999',
                'gid': '100',
                'firstname': 'Gregor',
                'lastname': 'von Laszewski',
                'phone': '812 ...',
                'e-mail': 'laszewski@gmail.com',
                'address': ['Indiana University', 'Bloomington, IN 47408']
                },
            'keys': {
                'name 1': 'file $HOME/.ssh/id_rsa.pub',
                'name 2': 'file $HOME/.ssh/id_rsa2.pub',
                'bla': 'key ssh-rsa AAAAB3.....zzzz keyname'
                },
            'projects': {
                'active': ['fg-82', 'fg-101'],
                'completed': ['fg-81', 'fg-102'],
                'default': 'fg-82'
                },
            'active': ['sierra-openstack-grizzly', 'india-openstack'],
            'default': 'sierra-openstack-grizzly'
            }

    @property
    def data(self):
        if self._data is None:
            self._load_data()
        return self._data

    @property
    def uid(self):
        return self.data['profile']['uid']

    @property
    def gid(self):
        return self.data['profile']['gid']

    @property
    def firstname(self):
        return self.data['profile']['firstname']

    @property
    def lastname(self):
        return self.data['profile']['lastname']

    @property
    def phone(self):
        return self.data['profile']['phone']

    @property
    def email(self):
        return self.data['profile']['e-mail']

    @property
    def address(self):
        return self.data['profile']['address']

    @property
    def keys(self):
        return self.data['keys']

    @property
    def projects(self):
        return self.data['projects']

    @property
    def activeclouds(self):
        return self.data['active']

    @property
    def defaultcloud(self):
        return self.data['default']

    @property
    def activeprojects(self):
        return self.data['projects']['active']

    @property
    def completedprojects(self):
        return self.data['projects']['completed']

    @property
    def defaultproject(self):
        return self.data['projects']['default']






















#! /usr/bin/env python
"""cm-manage config
------------
Command to generate rc files from our cloudmesh configuration files.

Usage:
  cm-manage config projects (list|?)
  cm-manage config [-f FILE] [-o OUT] [-p PROJECT] NAME [-]
  cm-manage config dump [--format=(yaml|dict)]
  cm-manage config init [-o OUT] [-u USER]
  cm-manage config list
  cm-manage --version
  cm-manage --help

This program generates form a YAML file containing the login
information for a cloud an rc file that can be used to later source
it.

Example:
  we assume the yaml file has an entry india-openstack

    cm-manage config -o novarc india-openstack
    source novarc

  This will create a novarc file and than you can source it.


     cm-manage config ? -

   Presents a selction of cloud choices and writes the choice into a
   file called ~/.futuregrid/novarc

Arguments:
  NAME name of the cloud

Options:
  -h --help            show this help message and exit

  -v --version         show version and exit

  -f NAME --file=NAME  the Name of the cloud to be specified, if ? a selection is presented

  -o OUT --out=OUT     writes the result in the specifide file

  -p PROJECT --project=PROJECT   selects a project (e.g. for eucalyptus which has project-specific environments)

  -u USER --user=USER  the user (login) name

  -d  --debug          debug

  -                    this option is a - at the end of the command. If data is written to a file it is also put out to stdout

"""
import yaml
from docopt import docopt
from cm_config import cm_config
import sys
import os
from ldap_user import ldap_user
from openstack_grizzly_cloud import openstack_grizzly_cloud

##### For testing
# import mock_keystone

debug = False

def DEBUG(label, var):
    if debug:
        print 70 * "-"
        print label 
        print 70 * " "
        print str(var)
        print 70 * "-"


def main():
    default_path = '.futuregrid/novarc'
    arguments = docopt(__doc__, version='0.8.1')

    DEBUG("arguments", arguments)
    
    home = os.environ['HOME']

    DEBUG("home", home)
    

    ######################################################################
    # This secion deals with handeling "cm config" related commands
    ######################################################################
    is_config = arguments['config'] != None

    if is_config:

        print arguments

        file = arguments['--file']
        try:
            config = cm_config(file)
            DEBUG("config", config)
        except IOError:
            print "%s: Configuration file '%s' not found" % ("CM ERROR", file)
            sys.exit(1)
        except (yaml.scanner.ScannerError, yaml.parser.ParserError) as yamlerror:
            print "%s: YAML error: %s, in configuration file '%s'" % ("CM ERROR", yamlerror, file)
            sys.exit(1)
        except:
            print "Unexpected error:", sys.exc_info()[0]
            sys.exit(1)

        name = arguments['NAME']


        if arguments['projects'] and arguments['list']:

            projects = config.data['cloudmesh']['projects']
            print yaml.dump(projects, default_flow_style=False, indent=4)
            sys.exit(0)

        if arguments['projects'] and arguments['?']:

            projects = config.data['cloudmesh']['projects']['active']

            print "Please select from the following:"
            print "0 - Cancel"
            selected = False
            choices = []
            while not selected:
                counter = 1
                for name in projects:
                    print counter, "-" "%20s" % name 
                    choices.append(name)
                    counter += 1
                print "Please select:"
                input = int(sys.stdin.readline())
                if input == 0:
                    print "Selection terminated"
                    sys.exit(0)
                selected = (input > 0) and (input < counter)
            print "Selected: ", input
            name = choices[input-1]
            print name

            sys.exit(0)

        if arguments['init'] or name == 'init':
            output = arguments['--out']
            username = arguments['--user'] or os.getenv('USER')
            config.userdata_handler = ldap_user
            config.cloudcreds_handler = openstack_grizzly_cloud
            ########### for testing #############################################################
            # config.cloudcreds_handler._client = mock_keystone.Client
            # config.cloudcreds_handler._client.mockusername = username
            # config.cloudcreds_handler._client.mocktenants = config.data['cloudmesh']['active']
            #####################################################################################
            config.initialize(username)
            config.write(output)
            sys.exit(0)

        if arguments['list'] or name == 'list':
            for name in config.keys():
                if 'cm_type' in config.data['cloudmesh'][name]:
                    print name, "(%s)" % config.data['cloudmesh'][name]['cm_type']
            sys.exit(0)

        if arguments['dump'] or name =='dump':
            format = arguments['--format']
            if format == 'yaml':
                print yaml.dump(config, default_flow_style=False, indent=4)
            elif format == 'dict' or format ==None:
                print config
            sys.exit(0)

        if name == '?':
            if file == None:
                arguments['--out'] = "%s/%s" % (home, default_path)
            print "Please select from the following:"
            print "0 - Cancel"
            selected = False
            choices = []
            while not selected:
                counter = 1
                for name in config.keys():
                    if 'cm_type' in config.data['cloudmesh']['clouds'][name]:
                        print counter, "-" "%20s" % name, "(%s)" % config.data['cloudmesh']['clouds'][name]['cm_type']
                        choices.append(name)
                        counter += 1
                print "Please select:"
                input = int(sys.stdin.readline())
                if input == 0:
                    print "Selection terminated"
                    sys.exit(0)
                selected = (input > 0) and (input < counter)
            print "Selected: ", input
            name = choices[input-1]

        output = arguments['--out']

        if name != None:
            if config.cloud(name)['cm_type'] == 'eucalyptus':
                if arguments['--project']:
                    project = arguments['--project']
                    if not project in config.cloud(name):
                        print "No such project %s defined in cloud %s." % (project, name)
                        sys.exit(1)
                else:
                    project = config.cloud_default(name, 'project') or config.projects('default')
                    if not project in config.cloud(name):
                        print "Default project %s not defined in cloud %s." % (project, name)
                        sys.exit(1)
                rc_func = lambda name: config.rc_euca(name, project)
            else:
                rc_func = config.rc

            try:
                result = rc_func(name)
            except:
                print "%s: The cloud '%s' can not be found" % ("CM ERROR", name)
                print "Try instead"
                for name in config.keys():
                    if 'cm_type' in config.data['cloudmesh'][name]:
                        print "    ", name

                sys.exit(1)

            if arguments["-"]:
                print result
            else:

                if output == None:
                    arguments['--out'] = "%s/%s" % (home, default_path)
                    output = arguments['--out']
                out = open(output, "w")
                out.write(result)
                out.close()
                os.system("cat " + str(output))

    ######################################################################
    # END "cm config" related commands
    ######################################################################

if __name__ == '__main__':
    main()



""" run with

nosetests -v --nocapture

or

nosetests -v

"""
import sys
#sys.path.insert(0, '..')

from cm_config import cm_config
from mock_user import mock_user
from mock_cloud import mock_cloud
from openstack_grizzly_cloud import openstack_grizzly_cloud
import mock_keystone
import json
import os
import warnings
import pprint
pp = pprint.PrettyPrinter(indent=4)

header = False

def HEADING(txt):
    if header:
        print
        print "#", 70 * '#'
        print "#", txt
        print "#", 70 * '#'

class Test_cloudmesh:


    filename = "credentials-example.yaml"
    #filename = None

    def setup(self):
        if self.filename == None:
            self.config = cm_config()
        else:
            self.config = cm_config(self.filename)

    def tearDown(self):
        pass

    def test01_print(self):
        print self.config

    def test02_active(self):
        HEADING("LIST ACTIVE PROJECTS")
        result = self.config.projects('active')
        assert result == ['fg-82', 'fg-101']
        
    def test03_completed(self):
        HEADING("LIST COMPLETED PROJECTS")
        result = self.config.projects('completed')		
        assert result == ['fg-81', 'fg-102']

    def test04_active(self):
	HEADING("LIST ACTIVE PROJECTS")
        result = self.config.projects('default')
        assert result == 'fg-82'

    def test05_india(self):
	HEADING("LIST India")
        result = self.config.get('india-openstack')
        assert result["OS_VERSION"] == "essex"

    def test06_keys_india_openstack(self):
	HEADING("KEY india-openstack")
        keys = self.config.keys()
        assert 'india-openstack' in keys

    def test07_keys_india_eucalyptus(self):
	HEADING("KEY india-eucalyptus")
	keys = self.config.keys()
        assert 'india-eucalyptus' in keys

    def test08_keys_grizzly_openstack(self):
	HEADING("KEY sierra-openstack-grizzly")
	keys = self.config.keys()
        assert 'sierra-openstack-grizzly' in keys

    def test09_keys_india_eucalyptus(self):
	HEADING("KEY azure")
	keys = self.config.keys()
        assert 'azure' in keys

    def test10_grizzly(self):
	HEADING("LIST GRIZZLY")
        result = self.config.get('sierra-openstack-grizzly')
	assert result["OS_VERSION"] == 'grizzly'

    def test11_grizzly(self):
	HEADING("LIST GRIZZLY EXPANDED")
        result = self.config.get('sierra-openstack-grizzly', expand=True)
	assert result["OS_VERSION"] == 'grizzly'

    def test12_clouds(self):
        HEADING("CLOUD")
        clouds = self.config.clouds()
        assert isinstance(clouds, dict)
        assert 'india-openstack' in clouds

    def test13_cloud(self):
        HEADING("CLOUD")
        india_cloud = self.config.cloud('india-openstack')
        assert isinstance(india_cloud, dict)
        assert 'cm_host' in india_cloud
        assert india_cloud['cm_host'] == 'india.futuregrid.org'

    def test14_cloud_default(self):
        HEADING("CLOUD")
        assert self.config.cloud_default('india-openstack', 'flavor') == 'm1.tiny'
        assert self.config.cloud_default('india-openstack', 'not defined') is None

    def test15_project_default(self):
        HEADING("PROJECT")
        project = self.config.projects('default')
        assert project == 'fg-82'

    def test16_write(self):
        HEADING("WRITE")
        warnings.filterwarnings('ignore', 'tempnam', RuntimeWarning)  # we open the file securely
        name = os.tempnam()
        print self.config
        self.config.write(name)
        print open(name,"rb").read()
        os.remove(name)

    def test17_key(self):
        HEADING("KEY")
        keys = self.config.userkeys()

        #print "DEFAULT>", self.config.userkeys('default')
        #print "TEST>", self.config.userkeys('test')

        print keys, keys['default'], keys['keylist'][keys['default']]
        assert ('default' in keys) and (keys['default'] in keys['keylist'])


    def test18_initialize(self):
        HEADING("INITIALIZATION")
        username = 'misterbojangles'
        self.config.userdata_handler = mock_user
        self.config.cloudcreds_handler = mock_cloud
        self.config.initialize(username)

        assert 'cloudmesh' in self.config.data
        assert len(self.config.data.keys()) == 1

        cmdata = self.config.data['cloudmesh']
        assert 'prefix' in cmdata
        assert 'profile' in cmdata
        assert 'username' in cmdata['profile']
        assert cmdata['profile']['username'] == username
        assert 'keys' in cmdata
        assert 'projects' in cmdata
        assert 'active' in cmdata
        assert 'default' in cmdata
        assert 'clouds' in cmdata
        assert 'security' in cmdata
        assert 'default' in cmdata['keys']
        assert 'india-openstack' in cmdata['clouds']
        assert 'sierra-openstack-grizzly' in cmdata['clouds']
        assert 'credentials' in cmdata['clouds']['sierra-openstack-grizzly']
        assert cmdata['clouds']['sierra-openstack-grizzly']['credentials']['OS_VERSION'] == 'grizzly'
        assert cmdata['clouds']['sierra-openstack-grizzly']['credentials']['OS_USERNAME'] == username
        assert cmdata['prefix'] == username

    def test19_openstack_grizzly(self):
        HEADING("INITIALIZATION")
        username = 'misterbojangles'
        self.config.userdata_handler = mock_user
        self.config.cloudcreds_handler = openstack_grizzly_cloud
        self.config.cloudcreds_handler._client = mock_keystone.Client
        self.config.cloudcreds_handler._client.mockusername = username
        self.config.cloudcreds_handler._client.mocktenants = self.config.data['cloudmesh']['active']
        self.config.initialize(username)
        cmdata = self.config.data['cloudmesh']
        assert cmdata['clouds']['sierra-openstack-grizzly']['credentials']['OS_VERSION'] == 'grizzly'
        assert cmdata['clouds']['sierra-openstack-grizzly']['credentials']['OS_USERNAME'] == username
        assert 'OS_PASSWORD' in cmdata['clouds']['sierra-openstack-grizzly']['credentials']
        assert 'project' in cmdata['clouds']['sierra-openstack-grizzly']['default']

    def test20_set_index_and_prefix(self):
        print
        print "INDEX:", self.config.index
        print "PREFIX:", self.config.prefix

        self.config.prefix = "hallo"
        self.config.index = "3"
        
        print "INDEX:", self.config.index
        print "PREFIX:", self.config.prefix
        print "NAME:", self.config.vmname


        self.config.incr()

        
        print "INDEX:", self.config.index
        print "PREFIX:", self.config.prefix
        print "NAME:", self.config.vmname

        assert self.config.index == 4 and self.config.prefix == "hallo"
